@@ -5,60 +5,54 @@ import importlib.metadata
 
 import click
 
-from tensorkube.constants import ADDON_NAME, REGION, Events, DEFAULT_NAMESPACE, get_cluster_name, CliColors
-from tensorkube.constants import get_mount_driver_role_name, get_mount_policy_name, get_base_login_url
-from tensorkube.helpers import create_mountpoint_driver_role_with_policy, sanitise_assumed_role_arn, track_event, \
-    sanitise_name
-from tensorkube.migration_service.migration_manager.migration_service import set_current_cli_version_to_cluster, \
-    migrate_tensorkube
-from tensorkube.services.aws_service import get_aws_account_id, get_credentials, get_bucket_name, get_aws_user_arn, \
-    get_aws_user_principal_arn, get_session_region
-from tensorkube.services.cloudformation_service import delete_cloudformation_stack, delete_launch_templates, \
-    cloudformation, get_validation_records_from_events, get_validation_records_with_wait
+from tensorkube.constants import REGION, Events, DEFAULT_NAMESPACE, get_cluster_name, CliColors, \
+    get_cfn_base_stack_name, TeardownType
+from tensorkube.constants import get_base_login_url
+from tensorkube.helpers import sanitise_assumed_role_arn, track_event, sanitise_name
+from tensorkube.migration_service.migration_manager.migration_service import migrate_tensorkube
+from tensorkube.services.aws_service import get_aws_account_id, get_aws_user_arn, \
+    get_aws_user_principal_arn, get_session_region, check_and_install_aws_cli, get_existing_bucket_name
+from tensorkube.services.cloudformation_service import get_validation_records_from_events, \
+    get_validation_records_with_wait, get_stack_status_from_stack_name, determine_teardown_type, CfnStackStatus
+from tensorkube.services.configure_service import legacy_configure, cfn_configure
 from tensorkube.services.dataset_service import upload_tensorkube_dataset, delete_tensorkube_dataset
-from tensorkube.services.deploy import Config, deploy_knative_service, DEFAULT_GPUS, DEFAULT_CPU, DEFAULT_MEMORY, \
+from tensorkube.services.deploy import deploy_knative_service, DEFAULT_GPUS, DEFAULT_CPU, DEFAULT_MEMORY, \
     DEFAULT_MIN_SCALE, DEFAULT_MAX_SCALE, DEFAULT_ENV, DEFAULT_SECRET, build_app, create_deployment_details
+from tensorkube.services.deploy_config_service import Config, run_deployment_config_pre_flight_checks
 from tensorkube.services.dynamodb_service import get_all_job_statuses, get_job_statuses
 from tensorkube.services.ecr_service import delete_all_tensorkube_ecr_repositories
-from tensorkube.services.eks_service import install_karpenter, apply_knative_crds, apply_knative_core, \
-    delete_knative_crds, delete_knative_core, delete_karpenter_from_cluster, apply_nvidia_plugin, create_eks_addon, \
-    delete_eks_addon, update_eks_kubeconfig, give_eks_cluster_access
-from tensorkube.services.eksctl_service import create_base_tensorkube_cluster_eksctl, delete_cluster
+from tensorkube.services.eks_service import update_eks_kubeconfig, give_eks_cluster_access
+
 from tensorkube.services.environment_service import create_new_environment, list_environments, delete_environment
-from tensorkube.services.filesystem_service import cleanup_filesystem_resources, configure_efs
+from tensorkube.services.filesystem_service import create_efs_volume_for_cluster_cfn, \
+    check_and_add_efs_volume_support_to_cluster, apply_storage_class, \
+    delete_efs_volume_cfn, delete_efs_volume_k8s_resources, list_efs_volumes, get_efs_filesystem_by_name
 from tensorkube.services.iam_service import create_iam_user, create_eks_access_policy, attach_policy_to_user, \
-    create_access_key, create_user_sqs_access_policy, create_user_s3_access_policy, create_user_dynamo_access_policy
-from tensorkube.services.iam_service import create_mountpoint_iam_policy, detach_role_policy, delete_role, delete_policy
-from tensorkube.services.istio import check_and_install_istioctl, install_istio_on_cluster, install_net_istio, \
-    install_default_domain, remove_domain_server, uninstall_istio_from_cluster
-from tensorkube.services.job_queue_service import deploy_job, create_cloud_resources_for_queued_job_support, queue_job, \
-    get_job_status, delete_all_job_resources, teardown_job_queue_support, is_existing_queued_job, \
-    create_sa_role_rb_for_job_sidecar
-from tensorkube.services.k8s_service import create_aws_secret, create_build_pv_and_pvc, create_secret, list_secrets, \
-    delete_secret, list_keda_scaled_jobs, add_user_to_k8s
-from tensorkube.services.karpenter_service import apply_karpenter_configuration
-from tensorkube.services.knative_service import enable_knative_selectors_pv_pvc_capabilities, cleanup_knative_resources, \
-    delete_knative_services
+    create_access_key, create_user_sqs_access_policy, create_user_s3_access_policy, create_user_dynamo_access_policy, \
+    has_configure_permissions
+from tensorkube.services.job_queue_service import deploy_job, queue_job, get_job_status, delete_all_job_resources, \
+    is_existing_queued_job
+from tensorkube.services.k8s_service import create_secret, list_secrets, delete_secret, list_keda_scaled_jobs, \
+    add_user_to_k8s
+from tensorkube.services.kubectl_utils import check_and_install_helm, check_and_install_kubectl
 from tensorkube.services.local_service import check_and_install_cli_tools
-from tensorkube.services.logging_service import configure_cloudwatch, teardown_cloudwatch
 from tensorkube.services.metapod_service import list_instances_in_cloud, start_devcontainer, purge_devcontainer, \
     pause_devcontainer, reset_cloud
 from tensorkube.services.rich_service import list_tensorkube_deployments, describe_deployment, display_deployment_logs, \
     ssh_into_deployed_service, display_secrets, list_tensorkube_datasets, display_job_logs, \
     delete_tensorkube_job_by_prefix, list_tensorkube_training_jobs, display_queued_jobs, get_tensorkube_training_job, \
-    prettify_aws_user_keys, display_specific_queued_job, display_dns_records_table, delete_deployment
-from tensorkube.services.s3_access_service import create_s3_access_to_pods
-from tensorkube.services.s3_service import create_s3_bucket, delete_s3_bucket
+    prettify_aws_user_keys, display_specific_queued_job, display_dns_records_table, delete_deployment, \
+    display_efs_volumes
+from tensorkube.services.teardown_service import legacy_teardown, cfn_teardown
 from tensorkube.services.tls_service import configure_certificate, get_certificate_stack_name, attach_domain_name, \
     validate_domain, check_if_a_domain_is_validated
 from tensorkube.services.train import axolotl_train, get_job_prefix_from_job_id
+from tensorkube.services.volumes_service import volume_name_satisfies_regex
 from tensorkube.tensorkube_token_server import run_server
-
-from tensorkube.services.nydus import get_nydus_image_url, install_nydus, delete_nydus
-
+from tensorkube.services.nydus import get_nydus_image_url
 from tensorkube.services.ecr_service import get_or_create_ecr_repository
 from tensorkube.services.version_service import print_tensorkube_version
-from tensorkube.services.build import configure_buildkit_irsa, delete_buildkit_irsa
+
 
 @click.group()
 @click.pass_context
@@ -137,7 +131,12 @@ def upgrade():
 
 
 @tensorkube.command()
-def configure():
+@click.option('--vpc-public-subnets', type=str, help='[Experimental] List of public subnets should be given if you want to create in existing VPC. If not given, a new VPC will be created. You should ignore this flag unless you want to create cluster in existing vpc. Tensorfuse recommends to create a new vpc for the tensorkube cluster instead of deploying in existing vpc. This is a enterprise feature. Reach out to tensorfuse for more information.')
+@click.option('--vpc-private-subnets', type=str, help='[Experimental] List of private subnets should be given if you want to create in existing VPC. If not given, a new VPC will be created.You should ignore this flag unless you want to create cluster in existing vpc. Tensorfuse recommends to create a new vpc for the tensorkube cluster instead of deploying in existing vpc. This is a enterprise feature. Reach out to tensorfuse for more information.')
+@click.option('--legacy', is_flag=True, help='Use the legacy configuration.')
+@click.option('--test', is_flag=True, help='Deploy using test configurations.')
+@click.option('--override-policy-checks', is_flag=True, help='NOT RECOMMENDED! ONLY USE IN RESEARCH SCENARIOS. Overrides policy checks.')
+def configure(vpc_public_subnets, vpc_private_subnets, legacy, test, override_policy_checks):
     """Configure the Tensorkube runtime on your private cloud
     """
     click.echo("Configuring the Tensorfuse runtime for your cloud...")
@@ -147,101 +146,94 @@ def configure():
         click.echo("You need to login to Tensorfuse to configure Tensorkube.")
         click.echo("Please login using the command: " + click.style("tensorkube login", fg='cyan'))
         return
+
+
+    if vpc_public_subnets and vpc_private_subnets:
+        if not legacy:
+            info_text = click.style("Configuring Tensorkube with existing VPCs is not supported in this version of configure. Please use the --legacy flag to use the legacy configuration.", fg=CliColors.WARNING.value)
+            click.echo(info_text)
+
+            end_time = time.time() * 1000
+            track_event(Events.CONFIGURE_END.value,
+                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "vpc_not_supported"})
+            return
     confirmation_txt = click.style("Confirmation: Tensorfuse sets up resources such as a Kubernetes cluster, load balancer, and cluster autoscaler to enable serverless GPUs and features like job queues. The estimated base cost for this setup is approximately ") + click.style("$300 per month", bold=True) + click.style(", billed by AWS. Please select \"Yes\" if you wish to proceed.")
     confirmation = click.confirm(confirmation_txt, default=False)
     if not confirmation:
         end_time = time.time() * 1000
         track_event(Events.CONFIGURE_END.value,
-                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "confirmation_declined"})    
+                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "confirmation_declined"})
         return
     session_region = get_session_region()
     if session_region != REGION:
         end_time = time.time() * 1000
         track_event(Events.CONFIGURE_END.value,
-                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "region_not_supported", "region": session_region})    
+                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "region_not_supported", "region": session_region})
         click.echo(click.style( f"Error: The current region is {session_region}. Tensorfuse currently supports only {REGION} region. Either switch your region to {REGION} or reach out to ", bold=True,
                         fg=CliColors.ERROR.value) + click.style( "founders@tensorfuse.io", italic=True, bold=True, fg=CliColors.ERROR.value) + click.style(" for support.", bold=True, fg=CliColors.ERROR.value))
         return
-    
-    if not check_and_install_cli_tools():
+
+    if override_policy_checks:
+        has_permissions = True
+        click.echo(click.style("You are overriding permissions and policy checks. You may run into permission issues", bold=True, fg=CliColors.WARNING.value))
+    else:
+        has_permissions = has_configure_permissions()
+    if not has_permissions:
+        end_time = time.time() * 1000
+        click.echo("Insufficient AWS permissions to run the configure command. Please contact your AWS admin.")
+        track_event(Events.CONFIGURE_END.value,
+                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "insufficient_permissions"})
         return
-    # TODO!: add helm annotations
 
-    # create cloudformation stack
-    cloudformation()
-    # create eks cluster
-    create_base_tensorkube_cluster_eksctl(cluster_name=get_cluster_name())
-    # install karpenter
-    install_karpenter()
-    # # apply karpenter configuration
-    apply_karpenter_configuration()
-    configure_cloudwatch()
-    #
-    # install istio networking plane
-    check_and_install_istioctl()
-    install_istio_on_cluster()
+    click.echo("All permissions are in place. Starting configure...")
+    if legacy:
+        try:
+            success = legacy_configure(vpc_public_subnets, vpc_private_subnets)
+            if not success:
+                return
 
-    # install knative crds
-    apply_knative_crds()
-    # install knative core
-    apply_knative_core()
+            end_time = time.time() * 1000
+            track_event(Events.CONFIGURE_END.value,
+                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
+            click.echo("Your tensorfuse cluster is ready and you are good to go.")
+        except Exception as e:
+            #TODO: add exception handling for ctrl+c
+            click.echo(f"Error while configuring Tensorfuse: {e}")
+            end_time = time.time() * 1000
+            track_event(Events.CONFIGURE_END.value,
+                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "error"})
+            return
+    else:
+        try:
+            check_and_install_aws_cli()
+            check_and_install_kubectl()
+            check_and_install_helm()
 
-    # install nvidia plugin
-    apply_nvidia_plugin()
-    #
-    # install net istio
-    install_net_istio()
+            cfn_configure(test)
 
-    # create s3 bucket for build
-    bucket_name = get_bucket_name()
-    create_s3_bucket(bucket_name)
+            status = get_stack_status_from_stack_name(get_cfn_base_stack_name())
+            if status != CfnStackStatus.CREATE_COMPLETE:
+                click.echo(f"Error while configuring Tensorfuse. Please check the cloudformation stack [{get_cfn_base_stack_name()}] for more details.")
+                end_time = time.time() * 1000
+                track_event(Events.CONFIGURE_ERROR.value,
+                            {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "error"})
+                return
 
-    
+            click.echo("Syncing config files for the tensorkube cluster...")
+            click.echo("Updating kubeconfig...")
+            update_eks_kubeconfig()
+            click.echo("Successfully updated the kubeconfig file.")
 
-    # create mountpoint policy to mount bucket to eks cluster
-    create_mountpoint_iam_policy(get_mount_policy_name(get_cluster_name()), bucket_name)
-
-    # create s3 csi driver role and attach mountpoint policy to it
-    create_mountpoint_driver_role_with_policy(cluster_name=get_cluster_name(), account_no=get_aws_account_id(),
-                                              role_name=get_mount_driver_role_name(get_cluster_name()),
-                                              policy_name=get_mount_policy_name(get_cluster_name()))
-
-    # create eks addon to mount s3 bucket to eks cluster
-    create_eks_addon(get_cluster_name(), ADDON_NAME, get_aws_account_id(),
-                     get_mount_driver_role_name(get_cluster_name()))
-
-    # create aws credentials cluster secret
-    # TODO!: figure out how to update credentials in case of token expiry
-    create_aws_secret(get_credentials())
-
-    # create pv and pvc claims for build
-    create_build_pv_and_pvc(bucket_name)
-
-    # update knative to use pod labels
-    enable_knative_selectors_pv_pvc_capabilities()
-    end_time = time.time() * 1000
-
-    # enable Network files system for the cluster
-    click.echo("Configuring EFS for the cluster...")
-    configure_efs()
-
-    # install keda, create related resources
-    create_cloud_resources_for_queued_job_support()
-    create_sa_role_rb_for_job_sidecar()
-
-    create_s3_access_to_pods()
-    
-    # configure buildkit irsa, so that pods can access ecr
-    configure_buildkit_irsa()
-
-    install_nydus()
-
-    # set current cli version to the cluster
-    set_current_cli_version_to_cluster()
-
-    track_event(Events.CONFIGURE_END.value,
-                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
-    click.echo("Your tensorfuse cluster is ready and you are good to go.")
+            end_time = time.time() * 1000
+            track_event(Events.CONFIGURE_END.value,
+                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
+            click.echo("Your tensorfuse cluster is ready and you are good to go!")
+        except Exception as e:
+            click.echo("Error while configuring Tensorfuse. {}".format(e))
+            end_time = time.time() * 1000
+            track_event(Events.CONFIGURE_END.value,
+                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "error"})
+            return
 
 
 @tensorkube.command()
@@ -283,148 +275,69 @@ def teardown():
         return
     else:
         # ask for  a secret phrase if given secret phrase is not correct then abort the teardown
-        secret_phrase = click.prompt(click.style("Please enter the secret phrase to confirm the teardown. Contact the founders for the phrase.", bold=True,fg=CliColors.ERROR.value), hide_input=True)
-        if secret_phrase == 'tensorsamagam':
+        secret_phrase = click.prompt(click.style("After this action you will no longer be able to use Tensorfuse. Please enter `accept` to confirm the teardown. ", bold=True,fg=CliColors.ERROR.value), hide_input=True)
+        if secret_phrase == 'accept':
             click.echo("Tear down confirmed.")
         else:
-            click.echo("Secret phrase is incorrect. Tear down aborted.")
+            click.echo("Phrase is incorrect. Tear down aborted.")
             return
 
     click.echo("Tearing down all resources...")
 
     # TODO?: add logic to delete any other resources
+    # TODO?: delete datasets bucket?
     click.echo("Deleting all ECR repositories...")
     delete_all_tensorkube_ecr_repositories()
 
-    click.echo("Deleting all job queue resources...")
-    try:
-        teardown_job_queue_support()
-    except Exception as e:
-        click.echo(f"Error while deleting job queue resources: {e}")
+    teardown_type = determine_teardown_type()
+    if teardown_type == TeardownType.LEGACY:
+        click.echo("Identified legacy configure. Starting legacy teardown...")
 
-    # delete all services
-    try:
-        delete_knative_services()
-    except Exception as e:
-        click.echo("Error while deleting Knative services.")
-    try:
-        cleanup_filesystem_resources()
-    except Exception as e:
-        click.echo(f"Error while cleaning up filesystem resources: {e}")
+        legacy_teardown()
 
-    # EKS addon
-    try:
-        click.echo("Deleting EKS addon...")
-        delete_eks_addon(get_cluster_name(), ADDON_NAME)
-    except Exception as e:
-        click.echo(f"Error while deleting EKS addon: {e}")
+        end_time = time.time() * 1000
+        track_event(Events.TEARDOWN_END.value,
+                    {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
+        click.echo("Tensorfuse has been successfully disconnected from your cluster.")
+    elif teardown_type == TeardownType.CFN:
+        click.echo("Identified configure. Starting teardown...")
+        train_bucket_name = get_existing_bucket_name("keda", "train")
+        if train_bucket_name:
+            click.echo(click.style(f"You train bucket, ", fg=CliColors.WARNING.value) +
+                       click.style(train_bucket_name, fg=CliColors.WARNING.value, bold=True, italic=True) +
+                       click.style(" will not be deleted as it might contain your models.", fg=CliColors.WARNING.value)
+                       )
+            click.echo(click.style("If you wish to delete the bucket, you must do so manually using the AWS console or using the following command:", fg=CliColors.WARNING.value))
+            click.echo(click.style(f"aws s3 rb s3://{train_bucket_name}", italic=True, fg=CliColors.WARNING.value))
+            click.echo(click.style("Use the ", fg=CliColors.WARNING.value)+
+                       click.style("--force", italic=True, fg=CliColors.WARNING.value)+
+                       click.style(" flag to delete any non-versioned objects in the bucket", fg=CliColors.WARNING.value)
+                       )
+            click.echo(click.style("Note: You will not be able to re-configure tensorkube unless you delete this bucket", bold=True, fg=CliColors.ERROR.value))
 
-    # teardown cloudwatch
-    try:
-        teardown_cloudwatch()
-    except Exception as e:
-        click.echo(f"Error while tearing down Cloudwatch: {e}")
+        cfn_teardown()
 
-    click.echo("Deleting Enviroments...")
-    try:
-        environments = list_environments()
-        for env in environments:
-            click.echo(f"Deleting environment: {env}")
-            delete_environment(env_name=env)
-    except Exception as e:
-        click.echo(f"Error while deleting environments: {e}")
+        # TODO: figure out where these are made and if we need to delete these in cfn
+        # click.echo("Deleting launch templates...")
+        # delete_launch_templates()
 
-    # Detach policy from role, delete role, delete policy
-    click.echo("Deleting mountpoint driver role and policy...")
-    click.echo("Detaching policy from role...")
-    try:
-        detach_role_policy(get_aws_account_id(), get_mount_driver_role_name(get_cluster_name()),
-                           get_mount_policy_name(get_cluster_name()))
-        click.echo("Deleting role...")
-        delete_role(get_mount_driver_role_name(get_cluster_name()))
-        click.echo("Deleting policy...")
-        delete_policy(get_aws_account_id(), get_mount_policy_name(get_cluster_name()))
-    except Exception as e:
-        click.echo(f"Error while deleting role and policy: {e}")
+        end_time = time.time() * 1000
+        track_event(Events.TEARDOWN_END.value,
+                    {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
 
-    # delete s3 bucket
-    click.echo("Deleting S3 bucket...")
-    try:
-        delete_s3_bucket(get_bucket_name())
-    except Exception as e:
-        click.echo(f"Error while deleting S3 bucket: {e}")
-
-    
-    click.echo("Deleting buildkit irsa...")
-    try:
-        delete_buildkit_irsa()
-    except Exception as e:
-        click.echo(f"Error while deleting buildkit irsa: {e}")
-    
-    click.echo("Uninstalling nydus helm charts...")
-    try:
-        delete_nydus()
-    except Exception as e:
-        click.echo(f"Error while uninstalling nydus resources: {e}")
-    
-    click.echo("Uninstalling domain server...")
-    try:
-        remove_domain_server()
-    except Exception as e:
-        click.echo(f"Error while uninstalling domain server: {e}")
-
-    click.echo("Uninstalling Knative resources")
-    try:
-        cleanup_knative_resources()
-    except Exception as e:
-        click.echo(f"Error while cleaning up Knative resources: {e}")
-
-    click.echo("Uninstalling and deleting Istio resources")
-    try:
-        uninstall_istio_from_cluster()
-    except Exception as e:
-        click.echo(f"Error while uninstalling Istio: {e}")
-    click.echo("Uninstalling Knative core")
-    try:
-        delete_knative_core()
-        click.echo("Uninstalling Knative CRDs")
-        delete_knative_crds()
-        click.echo("Successfully uninstalled Knative and Istio.")
-    except Exception as e:
-        click.echo(f"Error while uninstalling Knative: {e}")
-
-    # remove karpenter
-    click.echo("Uninstalling Karpenter...")
-    try:
-        delete_karpenter_from_cluster()
-        click.echo("Successfully uninstalled Karpenter.")
-    except Exception as e:
-        click.echo(f"Error while uninstalling Karpenter: {e}")
-    # delete cluster
-    try:
-        click.echo("Deleting cluster...")
-        delete_cluster()
-        click.echo("Successfully deleted cluster.")
-    except Exception as e:
-        click.echo(f"Error while deleting cluster.: {e}")
-    try:
-        # delete cloudformation stack
-        click.echo("Deleting cloudformation stack...")
-        delete_cloudformation_stack(get_cluster_name())
-        click.echo("Successfully deleted cloudformation stack.")
-    except Exception as e:
-        click.echo(f"Error while deleting cloudformation stack: {e}")
-
-    # delete launch templates
-    click.echo("Deleting launch templates...")
-    delete_launch_templates()
-
-    end_time = time.time() * 1000
-    track_event(Events.TEARDOWN_END.value,
-                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
-
-    click.echo("Successfully deleted launch templates.")
-    click.echo("Tensorfuse has been successfully disconnected from your cluster.")
+        # click.echo("Successfully deleted launch templates.")
+        stack_status = get_stack_status_from_stack_name(get_cfn_base_stack_name())
+        if stack_status == CfnStackStatus.DELETE_COMPLETE or stack_status is None:
+            click.echo("Tensorfuse has been successfully disconnected from your cluster.")
+        else:
+            click.echo(click.style(f"Error: Teardown incomplete. Please check the cloudformation stack [{get_cfn_base_stack_name()}] for more details.", fg=CliColors.ERROR.value))
+            click.echo(click.style("Stack deletion incomplete. There may be dangling resources. Please check your AWS console for more details.", fg=CliColors.ERROR.value, bold=True))
+            click.echo(click.style("If you are unable to resolve the issue, please reach out to the tensorfuse team for support", fg=CliColors.ERROR.value))
+    else:
+        click.echo("Unknown teardown type. Please contact support.")
+        end_time = time.time() * 1000
+        track_event(Events.TEARDOWN_END.value,
+                    {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "unknown_teardown_type"})
 
 
 @tensorkube.command()
@@ -449,7 +362,7 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
     Deploy your containerized application on Tensorkube. This command requires
     a dockerfile to be present in the current directory.
     """
-
+    click.echo('Preparing configurations for deployment...')
     if env == "default":
         env = DEFAULT_ENV
     try:
@@ -458,12 +371,19 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
         click.echo(f"Failed to read config.yaml file: {e}")
         return
 
-    config.set_if_not_exist('gpus', gpus).set_if_not_exist('gpu_type', gpu_type).set_if_not_exist('cpu',
-                                                                                                  cpu).set_if_not_exist(
-        'memory', memory).set_if_not_exist('min_scale', min_scale).set_if_not_exist('max_scale',
-                                                                                    max_scale).set_if_not_exist('env',
-                                                                                                                env).set_if_not_exist(
-        'github_actions', github_actions).set_if_not_exist('secret', secret).set_if_not_exist('domain', domain_name).set_if_not_exist('efs', efs).set_if_not_exist('concurrency', concurrency).set_if_not_exist('port', port)
+    config.set_if_not_exist('gpus', gpus)\
+        .set_if_not_exist('gpu_type', gpu_type)\
+        .set_if_not_exist('cpu', cpu)\
+        .set_if_not_exist('memory', memory)\
+        .set_if_not_exist('min_scale', min_scale)\
+        .set_if_not_exist('max_scale',max_scale)\
+        .set_if_not_exist('env',env)\
+        .set_if_not_exist('github_actions', github_actions)\
+        .set_if_not_exist('secret', secret)\
+        .set_if_not_exist('domain_name', domain_name)\
+        .set_if_not_exist('efs', efs)\
+        .set_if_not_exist('concurrency', concurrency)\
+        .set_if_not_exist('port', port)
 
     # flags validation
     if config.get('gpus') > 0:
@@ -471,9 +391,10 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
             click.echo(click.style("Error: GPU type is required when GPUs are specified.", fg=CliColors.ERROR.value))
             return
     # TODO add domain validation check to see if the domain is configured properly
-    if domain_name is not None:
-        is_domain_valid = validate_domain(domain=domain_name)
-        is_domain_validated = check_if_a_domain_is_validated(domain=domain_name)
+    if config.get('domain_name') is not None:
+        click.echo(click.style("Validating domain name...", fg=CliColors.INFO.value))
+        is_domain_valid = validate_domain(domain=config.get('domain_name'))
+        is_domain_validated = check_if_a_domain_is_validated(domain=config.get('domain_name'))
         if not (is_domain_valid and is_domain_validated):
             click.echo(click.style("Error: Invalid domain name.", fg=CliColors.ERROR.value))
             return
@@ -488,6 +409,7 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
     if config.get('gpus') not in [0, 1, 4, 8]:
         click.echo('Error: Invalid number of GPUs. Only supported values are 0, 1, 4, and 8.')
         return
+    click.echo(click.style("Preparing image details", fg=CliColors.INFO.value))
     depl_secrets, env_namespace, image_tag, cwd, sanitised_project_name = create_deployment_details(secret=config.get('secret'),
                                                                                                   env=config.get('env'))
     if not image_tag:
@@ -499,29 +421,35 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
     image_url = f"{repo_url}:{image_tag}"
     
     convert_to_nydus_image = True  if config.get('gpus') > 0 and not enable_efs else False
+    click.echo(click.style("Starting the build process...", fg=CliColors.INFO.value))
+
+    click.echo("Running pre-flight checks...")
+    checks_successful = run_deployment_config_pre_flight_checks(config)
+    if not checks_successful:
+        click.echo("Deployment checks failed. Please check the logs for more details.")
+        return
+
+
     pod_status = build_app(env=config.get('env'), image_tag=image_tag, sanitised_project_name=sanitised_project_name,
-                           upload_to_nfs=enable_efs, image_url=image_url, convert_to_nydus_image=convert_to_nydus_image)
+                           upload_to_nfs=enable_efs, image_url=image_url, convert_to_nydus_image=convert_to_nydus_image, secrets=config.get('secret'))
     if config.get('gpus') > 0:
         image_url = get_nydus_image_url(image_url=image_url)
     if enable_efs:
         image_url = image_tag
     deploy_knative_service(env=config.get('env'), gpus=config.get('gpus'),
                            gpu_type=config.get('gpu_type'), cpu=config.get('cpu'), memory=config.get('memory'),
-                           min_scale=config.get('min_scale'), max_scale=config.get('max_scale'),concurrency= config.get('concurrency'), pod_status=pod_status,
-                           cwd=cwd, sanitised_project_name=sanitised_project_name, image_url=image_url, secrets=depl_secrets,
-                           enable_efs=enable_efs, domain=config.get('domain'),github_actions=config.get('github_actions'), readiness = config.get('readiness'), port = config.get('port'))
+                           min_scale=config.get('min_scale'), max_scale=config.get('max_scale'),
+                           concurrency= config.get('concurrency'), pod_status=pod_status, cwd=cwd,
+                           sanitised_project_name=sanitised_project_name, image_url=image_url, secrets=depl_secrets,
+                           enable_efs=enable_efs, domain=config.get('domain_name'), github_actions=config.get('github_actions'),
+                           startup_probe = config.get('startup_probe'), liveness_probe = config.get('liveness'),
+                           readiness = config.get('readiness'), port = config.get('port'), volumes=config.get('volumes'))
 
     end_time = time.time() * 1000
     track_event(Events.DEPLOY_END.value,
                 {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
 
 
-# @tensorkube.command()
-# def delete_project():
-#     click.echo("Deleting the project resources...")
-#     # TODO!: add logic to delete the ecr repository, s3 folder, build job, and any other resources
-#     click.echo("Successfully deleted the project resources.")
-
 
 @tensorkube.command()
 def get_permissions_command():
@@ -719,7 +647,7 @@ def create(secret_name, key_value_pairs, env, force):
     """Create a new secret in the specified environment. Use `--force` to overwrite an existing one.
     """
     try:
-        key_values = dict(pair.split('=') for pair in key_value_pairs)
+        key_values = dict(pair.split('=',1) for pair in key_value_pairs)
     except Exception as e:
         click.echo(
             click.style("Error: Invalid key-value pairs. Please provide key-value pairs in the format KEY=value.",
@@ -769,7 +697,7 @@ def list(env):
 @tensorkube.command()
 def test():
     click.echo("Running tests... for someone")
-    
+
 
 
 @tensorkube.command()
@@ -1024,7 +952,7 @@ def deploy(name, gpus, gpu_type, cpu, memory, secret, max_scale, update, efs):
     repo_url = get_or_create_ecr_repository(sanitised_project_name)
     image_url = f"{repo_url}:{image_tag}"
     convert_to_nydus_image = True if gpus > 0 and not efs else False
-    pod_status = build_app(env=env, image_tag=image_tag, sanitised_project_name=sanitised_job_name, upload_to_nfs=efs,image_url=image_url, convert_to_nydus_image=convert_to_nydus_image)
+    pod_status = build_app(env=env, image_tag=image_tag, sanitised_project_name=sanitised_job_name, upload_to_nfs=efs,image_url=image_url, convert_to_nydus_image=convert_to_nydus_image, secrets=secret)
     if gpus > 0:
         image_url = get_nydus_image_url(image_url=image_url)
     if efs:
@@ -1185,4 +1113,85 @@ def get_principal_arn():
 
 @tensorkube.command()
 def version():
-    print_tensorkube_version()
\ No newline at end of file
+    print_tensorkube_version()
+
+
+@tensorkube.group()
+def volume():
+    """Manage volumes for your Tensorkube cluster.
+    """
+    pass
+
+@volume.command()
+@click.option('--name', required=True, help='Name of the volume.')
+@click.option('--type', type=click.Choice(['efs'], case_sensitive=False),
+              help='Type of volume.', required=True)
+def create(name, type):
+    """Create a new volume for your Tensorkube cluster.
+    """
+    if not volume_name_satisfies_regex(name):
+        click.echo(click.style("Error: Volume name can only contain characters that are lowercase, numbers or hyphens. It must start and and with an alphanumeric character.", fg=CliColors.ERROR.value))
+        return
+
+    if type == 'efs':
+        click.echo("Checking if EFS volume already exists...")
+        filesystem = get_efs_filesystem_by_name(name)
+        if filesystem is not None:
+            click.echo(click.style(f"EFS volume with name '{name}' already exists. Skipping creation.", fg=CliColors.INFO.value))
+            return
+
+        click.echo("Creating EFS volume...")
+        click.echo("Creating EFS volume support for the cluster...")
+        support_created = check_and_add_efs_volume_support_to_cluster()
+        if not support_created:
+            click.echo(click.style("Error: Failed to create EFS volume support for the cluster.", fg=CliColors.ERROR.value))
+            return
+
+        volume_id = create_efs_volume_for_cluster_cfn(name)
+        if not volume_id:
+            click.echo(click.style("Error: Failed to create EFS volume.", fg=CliColors.ERROR.value))
+            return
+
+        apply_storage_class()
+    else:
+        click.echo(click.style("Error: Invalid volume type.", fg=CliColors.ERROR.value))
+
+
+@volume.command()
+@click.option('--name', required=True, help='Name of the volume.')
+@click.option('--type', type=click.Choice(['efs'], case_sensitive=False),
+              help='Type of volume.', required=True)
+def delete(name, type):
+    """Delete a volume for your Tensorkube cluster.
+    """
+    confirmation_text = f"Are you sure you want to delete volume {name}? This action cannot be undone. Any deployments using this might fail or freeze. Please ensure that all the deployments using this volume are deleted before proceeding."
+    confirmation = click.confirm(click.style(confirmation_text, bold=True, fg=CliColors.ERROR.value), default=False)
+    if not confirmation:
+        click.echo("Deletion aborted")
+        return
+
+    if type == 'efs':
+        click.echo("Deleting EFS volume...")
+        volume_deleted = delete_efs_volume_cfn(name)
+        if not volume_deleted:
+            click.echo(click.style("Error: Failed to delete EFS volume.", fg=CliColors.ERROR.value))
+            return
+        k8s_resources_deleted = delete_efs_volume_k8s_resources(name)
+        if not k8s_resources_deleted:
+            click.echo(click.style("Error: Failed to delete EFS volume resources.", fg=CliColors.ERROR.value))
+            return
+        click.echo("EFS volume deleted successfully.")
+    else:
+        click.echo(click.style("Error: Invalid volume type.", fg=CliColors.ERROR.value))
+
+
+@volume.command()
+@click.option('--type', type=click.Choice(['efs'], case_sensitive=False))
+def list(type):
+    """List all the volumes in the Tensorkube cluster.
+    """
+    if type == 'efs':
+        volume_list = list_efs_volumes()
+        display_efs_volumes(volume_list)
+    else:
+        click.echo(click.style("Error: Invalid volume type.", fg=CliColors.ERROR.value))
