@@ -1,15 +1,17 @@
 import enum
-import pprint
-import threading
 import time
+from typing import List, Dict
 from typing import Tuple, Optional
 
 import botocore.exceptions
 import click
 from pkg_resources import resource_filename
 
-from tensorkube.services.aws_service import get_cloudformation_client, get_ec2_client, get_iam_client
+from tensorkube.constants import DEFAULT_CAPABILITIES, TENSORKUBE_MACRO_NAME, get_cfn_base_stack_name, TeardownType
+from tensorkube.services.aws_service import get_cloudformation_client, get_ec2_client, get_iam_client, \
+    get_aws_account_id
 from tensorkube.constants import get_cluster_name, TENSORFUSE_STRING, CREATED_BY_TAG, CLUSTER_NAME_TAG
+from tensorkube.services.iam_service import delete_policy
 
 
 class CfnStackStatus(enum.Enum):
@@ -55,56 +57,52 @@ def get_stack_status_from_stack_name(stack_name: str) -> CfnStackStatus:
         raise
 
 
-def create_cloudformation_stack()-> Tuple[bool, bool]:
+def create_cloudformation_stack(template_file_path: str, stack_name: str, parameters: Optional[List[Dict]] = None,
+                                capabilities: List[str] = DEFAULT_CAPABILITIES)-> Tuple[bool, bool]:
     cloudformation_client = get_cloudformation_client()
-
-    # stack name
-    cluster_name = get_cluster_name()
-    parameters = [{"ParameterKey": "ClusterName", 'ParameterValue': cluster_name}]
-
-    file_name = resource_filename('tensorkube', 'configurations/karpenter_cloudformation.yaml')
+    file_name = resource_filename('tensorkube', template_file_path)
 
     try:
-        stack = cloudformation_client.describe_stacks(StackName=cluster_name)
+        stack = cloudformation_client.describe_stacks(StackName=stack_name)
         if stack['Stacks'][0]['StackStatus'] == 'CREATE_COMPLETE':
-            click.echo(f"Stack {cluster_name} already exists. Skipping stack creation")
+            click.echo(f"Stack {stack_name} already exists. Skipping stack creation")
             creation_queued = False
             in_created_state = True
             return creation_queued, in_created_state
         elif stack['Stacks'][0]['StackStatus'] == 'CREATE_IN_PROGRESS':
-            click.echo(f"Stack {cluster_name} already exists and is in creation state. Please wait for "
+            click.echo(f"Stack {stack_name} already exists and is in creation state. Please wait for "
                        f"creation to complete.")
             creation_queued = True
             in_created_state = False
             return creation_queued, in_created_state
         elif stack['Stacks'][0]['StackStatus'] == 'ROLLBACK_COMPLETE':
-            click.echo(f"Stack {cluster_name} already exists but is in rollback state. Please delete the stack "
+            click.echo(f"Stack {stack_name} already exists but is in rollback state. Please delete the stack "
                        f"and recreate it.")
             creation_queued = False
             in_created_state = False
             return creation_queued, in_created_state
         elif stack['Stacks'][0]['StackStatus'] == 'DELETE_COMPLETE':
-            # use the configurations/karpenter_cloudformation.yaml in the library
+            # use the configurations/cloudformation/karpenter_cloudformation.yaml in the library
             with open(file_name) as file:
                 template = file.read()
-            response = cloudformation_client.create_stack(StackName=cluster_name, TemplateBody=template, Parameters=parameters,
-                                                          Capabilities=["CAPABILITY_NAMED_IAM"])
+            response = cloudformation_client.create_stack(StackName=stack_name, TemplateBody=template, Parameters=parameters,
+                                                          Capabilities=capabilities)
             creation_queued = True
             in_created_state = False
             return creation_queued, in_created_state
         else:
-            click.echo(f"Stack {cluster_name} already exists but not in created state. Either wait for "
+            click.echo(f"Stack {stack_name} already exists but not in created state. Either wait for "
                        f"creation to complete or delete stack to recreate it.")
             creation_queued = False
             in_created_state = False
             return creation_queued, in_created_state
     except botocore.exceptions.ClientError as e:
         if 'does not exist' in str(e):
-            # use the configurations/karpenter_cloudformation.yaml in the library
+            # use the configurations/cloudformation/karpenter_cloudformation.yaml in the library
             with open(file_name) as file:
                 template = file.read()
-            response = cloudformation_client.create_stack(StackName=cluster_name, TemplateBody=template, Parameters=parameters,
-                                                          Capabilities=["CAPABILITY_NAMED_IAM"])
+            response = cloudformation_client.create_stack(StackName=stack_name, TemplateBody=template, Parameters=parameters,
+                                                          Capabilities=capabilities)
             creation_queued = True
             in_created_state = False
             return creation_queued, in_created_state
@@ -122,7 +120,7 @@ def delete_role_and_attached_policies(iam, role_name):
         iam.detach_role_policy(RoleName=role_name, PolicyArn=policy['PolicyArn'])
         if get_cluster_name() in policy['PolicyName']:
             # Delete the policy
-            iam.delete_policy(PolicyArn=policy['PolicyArn'])
+            delete_policy(account_no=get_aws_account_id(), policy_name=policy['PolicyName'])
 
     # Delete the role
     iam.delete_role(RoleName=role_name)
@@ -173,6 +171,32 @@ def delete_cloudformation_stack(stack_name):
     return response
 
 
+
+def delete_tensorkube_base_stack():
+    cloudformation_client = get_cloudformation_client()
+    stack_name = get_cfn_base_stack_name()
+
+    try:
+        cloudformation_client.describe_stacks(StackName=stack_name)
+    except botocore.exceptions.ClientError as e:
+        if 'does not exist' in str(e):
+            print(f'Stack {stack_name} does not exist.')
+            return
+        else:
+            raise
+
+    response = cloudformation_client.delete_stack(StackName=stack_name)
+    # Create a waiter to wait for the stack to be deleted
+    waiter = cloudformation_client.get_waiter('stack_delete_complete')
+
+    # Start streaming the stack events in a separate thread
+    stream_stack_events(stack_name)
+
+    # Wait for the stack to be deleted
+    waiter.wait(StackName=stack_name, WaiterConfig={'MaxAttempts': 30})
+    return response
+
+
 def delete_launch_templates():
     ec2_client = get_ec2_client()
     cluster_name = get_cluster_name()
@@ -212,9 +236,13 @@ def cloudformation():
     """Create a cloudformation stack."""
     click.echo("Creating cloudformation stack...")
     # create_cloudformation_stack()
-    creation_queued, in_created_state = create_cloudformation_stack()
+    stack_name = get_cluster_name()
+    parameters = [{"ParameterKey": "ClusterName", 'ParameterValue': get_cluster_name()}]
+    creation_queued, in_created_state = create_cloudformation_stack(
+        template_file_path="configurations/cloudformation/karpenter_cloudformation.yaml", stack_name=stack_name,
+        parameters=parameters)
     if creation_queued:
-        stream_stack_events(get_cluster_name())
+        stream_stack_events(stack_name)
     click.echo("Cloudformation stack created.")
 
 
@@ -245,6 +273,7 @@ def deploy_generic_cloudformation_stack(stack_name: str, template_body: str, par
             click.echo(f'Stack status: {current_status}')
 
             if current_status == CfnStackStatus.CREATE_COMPLETE.value:
+                click.echo(f"Stack {stack_name} already exists. Skipping stack creation.")
                 return True
             elif current_status in [CfnStackStatus.CREATE_IN_PROGRESS.value,
                                     CfnStackStatus.UPDATE_IN_PROGRESS.value]:
@@ -274,6 +303,7 @@ def deploy_generic_cloudformation_stack(stack_name: str, template_body: str, par
                     EnableTerminationProtection=enable_termination_protection,
                     Tags=tags
                 )
+                click.echo(f"Stack {stack_name} creation initiated.")
 
                 # Wait for stack creation
                 if should_wait:
@@ -351,3 +381,99 @@ def get_validation_records_with_wait(stack_name: str, timeout: int = 300, poll_i
 
     click.echo(click.style("Timeout reached while waiting for DNS validation records.", fg='red'))
     return None
+
+
+def is_existing_tk_macro(macro_name: str = TENSORKUBE_MACRO_NAME) -> bool:
+    cloudformation_client = get_cloudformation_client()
+    paginator = cloudformation_client.get_paginator('list_stack_resources')
+
+    # Iterate through all stacks
+    stacks = cloudformation_client.list_stacks(StackStatusFilter=['CREATE_COMPLETE'])['StackSummaries']
+    for stack in stacks:
+        stack_name = stack['StackName']
+        pages = paginator.paginate(StackName=stack_name)
+        for page in pages:
+            for resource in page['StackResourceSummaries']:
+                if (resource['ResourceType'] == 'AWS::CloudFormation::Macro'
+                        and resource['PhysicalResourceId'] == macro_name):
+                    print(resource)
+                    print(f"Macro '{macro_name}' exists in stack '{stack_name}'.")
+                    return True
+
+    print(f"Macro '{macro_name}' does not exist.")
+    return False
+
+
+def determine_teardown_type():
+    cloudformation_client = get_cloudformation_client()
+    paginator = cloudformation_client.get_paginator('list_stacks')
+    stack_status_filter = [
+        'CREATE_IN_PROGRESS', 'CREATE_COMPLETE', 'ROLLBACK_IN_PROGRESS', 'ROLLBACK_COMPLETE',
+        'DELETE_IN_PROGRESS', 'UPDATE_IN_PROGRESS', 'UPDATE_COMPLETE', 'UPDATE_ROLLBACK_IN_PROGRESS',
+        'UPDATE_ROLLBACK_COMPLETE'
+    ]
+
+    for page in paginator.paginate(StackStatusFilter=stack_status_filter):
+        click.echo("Identifying stack type...")
+
+        stacks = page['StackSummaries']
+
+        for stack in stacks:
+            if stack['StackName'] == get_cluster_name():
+                return TeardownType.LEGACY
+
+            if stack['StackName'] == get_cfn_base_stack_name():
+                return TeardownType.CFN
+
+
+    return TeardownType.UNKNOWN
+
+
+def get_cfn_stack_export_values(export_names: List[str]) -> Dict[str, str]:
+    cloudformation_client = get_cloudformation_client()
+    paginator = cloudformation_client.get_paginator('list_exports')
+    export_values = {}
+
+    for page in paginator.paginate():
+        for export in page['Exports']:
+            if export['Name'] in export_names:
+                export_values[export['Name']] = export['Value']
+
+    return export_values
+
+
+def delete_generic_cfn_stack(stack_name: str):
+    cloudformation_client = get_cloudformation_client()
+    try:
+        cloudformation_client.describe_stacks(StackName=stack_name)
+    except botocore.exceptions.ClientError as e:
+        if 'does not exist' in str(e):
+            click.echo(f'Stack {stack_name} does not exist.')
+            return
+        else:
+            raise
+
+    response = cloudformation_client.delete_stack(StackName=stack_name)
+    # Create a waiter to wait for the stack to be deleted
+    waiter = cloudformation_client.get_waiter('stack_delete_complete')
+
+    stream_stack_events(stack_name)
+
+    # Wait for the stack to be deleted
+    waiter.wait(StackName=stack_name, WaiterConfig={'MaxAttempts': 30})
+    return response
+
+
+def update_generic_cloudformation_stack(stack_name: str, template_file_path: str, parameters: List[Dict[str, str]],
+                                        capabilities: List[str] = DEFAULT_CAPABILITIES):
+    cloudformation_client = get_cloudformation_client()
+    file_name = resource_filename('tensorkube', template_file_path)
+
+    with open(file_name) as file:
+        template = file.read()
+
+    response = cloudformation_client.update_stack(StackName=stack_name, TemplateBody=template, Parameters=parameters,
+                                                  Capabilities=capabilities)
+
+    stream_stack_events(stack_name)
+    return response
