@@ -69,7 +69,7 @@ def install_karpenter():
     # Logout from helm registry
     logout_command = ["helm", "registry", "logout", "public.ecr.aws"]
     try:
-        subprocess.run(logout_command, check=True)
+        subprocess.run(logout_command, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
     except Exception as e:
         pass
 
