@@ -49,14 +49,17 @@ def create_new_environment(env_name: str):
     configure_buildkit_irsa(env_name=env_name)
 
 
-def delete_environment(env_name: str):
+def delete_environment(env_name: str, is_teardown = False):
     # delete the namespace for the environment
     # delete the s3 bucket for the environment
     # delete the NFS mounts for the environment
     if env_name is None:
         click.echo("Please provide a valid environment name to delete")
         return
-    if env_name in TENSORFUSE_NAMESPACES or env_name == DEFAULT_NAMESPACE:
+    if env_name in TENSORFUSE_NAMESPACES:
+        click.echo("Cannot delete a system namespace")
+        return
+    if env_name == DEFAULT_NAMESPACE and (not is_teardown):
         click.echo("Cannot delete a system namespace")
         return
     if env_name not in list_environments():
