@@ -4,12 +4,16 @@ from typing import Optional
 import boto3
 import click
 import yaml
+import subprocess
 from kubernetes import config, client, utils
 from pkg_resources import resource_filename
 
 from tensorkube.constants import NAMESPACE, REGION, get_efs_service_account_name, get_efs_role_name, get_cluster_name, \
-    DEFAULT_NAMESPACE, get_efs_security_group_name
+    DEFAULT_NAMESPACE, get_efs_security_group_name, CliColors
 from tensorkube.services.aws_service import get_aws_account_id
+from tensorkube.services.cloudformation_service import create_cloudformation_stack, stream_stack_events, \
+    get_stack_status_from_stack_name, CfnStackStatus, deploy_generic_cloudformation_stack, get_cfn_stack_export_values, \
+    delete_generic_cfn_stack
 from tensorkube.services.eks_service import get_cluster_oidc_issuer_url, get_eks_cluster_vpc_config, get_vpc_cidr, \
     get_security_group_id_by_name
 from tensorkube.services.iam_service import delete_iam_role
@@ -34,6 +38,18 @@ def get_efs_pvc_name(env: Optional[str] = None):
         return f'efs-pvc-env-{env}'
     return 'efs-pvc'
 
+def get_efs_vol_cfn_stack_name(volume_name: str):
+    return f'{get_cluster_name()}-efs-volume-{volume_name}'
+
+def get_efs_vol_pv_name(volume_name: str, namespace: str = DEFAULT_NAMESPACE):
+    if not namespace:
+        namespace = DEFAULT_NAMESPACE
+    return f'efs-pv-{volume_name}-{namespace}'
+
+def get_efs_vol_pvc_name(volume_name: str, namespace: str = DEFAULT_NAMESPACE):
+    if not namespace:
+        namespace = DEFAULT_NAMESPACE
+    return f'efs-pvc-{volume_name}-{namespace}'
 
 def create_efs_driver_role(account_no: str, role_name: str, oidc_issuer_url: str, namespace: str,
                            service_account_name: str):
@@ -219,6 +235,35 @@ def get_az_from_subnet(subnet_id):
     return response['Subnets'][0]['AvailabilityZone']
 
 
+def is_public_subnet(subnet_id, region='us-east-1'):
+    ec2 = boto3.client('ec2', region_name=region)
+
+    # Get subnet details
+    subnet = ec2.describe_subnets(SubnetIds=[subnet_id])['Subnets'][0]
+
+    # Get the route table associated with the subnet
+    route_table_associations = ec2.describe_route_tables(
+        Filters=[{'Name': 'association.subnet-id', 'Values': [subnet_id]}]
+    )['RouteTables']
+
+    if not route_table_associations:
+        return "Unknown (No Route Table Associated)"
+
+    route_table = route_table_associations[0]  # Assume first route table
+
+    # Check for routes in the route table
+    for route in route_table.get('Routes', []):
+        if route.get('DestinationCidrBlock') == '0.0.0.0/0':
+            gateway_id = route.get('GatewayId')
+            nat_gateway_id = route.get('NatGatewayId')
+
+            if gateway_id and 'igw-' in gateway_id:
+                return True
+            elif nat_gateway_id and 'nat-' in nat_gateway_id:
+                return False
+    return False
+
+
 def create_efs_mount_targets(filesystem_id, subnets, security_group_id):
     efs_client = boto3.client('efs')
 
@@ -255,6 +300,139 @@ def configure_efs_for_the_cluster(env: Optional[str] = None):
     create_efs_mount_targets(file_system_id, subnets, security_group_id)
 
 
+def check_and_add_efs_volume_support_to_cluster() -> bool:
+    efs_volume_support_stack_name = f'{get_cluster_name()}-volume-support-efs'
+
+    template_path = resource_filename('tensorkube', 'configurations/cloudformation/volumes/efs/efs_volume_support.yaml')
+    template_body = open(template_path, 'r').read()
+    oidc_provider_url = get_cluster_oidc_issuer_url(get_cluster_name())
+    deployed = deploy_generic_cloudformation_stack(
+        stack_name=efs_volume_support_stack_name,
+        template_body=template_body,
+        parameters=[{'ParameterKey': 'ClusterName', 'ParameterValue': get_cluster_name()},
+                    {'ParameterKey': 'OIDCProviderURL', 'ParameterValue': oidc_provider_url}],
+        capabilities=["CAPABILITY_NAMED_IAM", "CAPABILITY_AUTO_EXPAND"],
+        should_wait=False
+    )
+
+    if deployed:
+        status = get_stack_status_from_stack_name(efs_volume_support_stack_name)
+        if status != CfnStackStatus.CREATE_COMPLETE:
+            click.echo("Streaming events...")
+            stream_stack_events(efs_volume_support_stack_name)
+
+        status = get_stack_status_from_stack_name(efs_volume_support_stack_name)
+        if not status:
+            return False
+        elif status == CfnStackStatus.CREATE_COMPLETE:
+            click.echo("EFS volume support added to the cluster")
+            return True
+        else:
+            return False
+    else:
+        return False
+
+
+def create_efs_volume_for_cluster_cfn(volume_name: str):
+    cluster_name = get_cluster_name()
+    vpc_config = get_eks_cluster_vpc_config(cluster_name)
+    vpc_id = vpc_config['vpcId']
+    subnets = vpc_config['subnetIds']
+
+    parameters = [{'ParameterKey': 'VolumeName', 'ParameterValue': volume_name},
+                  {'ParameterKey': 'ClusterName', 'ParameterValue': cluster_name},
+                  {'ParameterKey': 'VpcId', 'ParameterValue': vpc_id},
+                  {'ParameterKey': 'CidrRange', 'ParameterValue': get_vpc_cidr(vpc_id)}]
+
+    click.echo("Identifying subnets...")
+    for subnet in subnets:
+        az = get_az_from_subnet(subnet)
+        is_public = is_public_subnet(subnet)
+        if is_public:
+            continue
+        subnet_type = 'Private'
+        subnet_zone = az[-1].upper()
+        paramter_key = f"ZONE{subnet_zone}{subnet_type}SubnetId"
+        parameters.append({'ParameterKey': paramter_key, 'ParameterValue': subnet})
+
+    click.echo("Subnets identified.")
+    click.echo("Creating EFS volume CloudFormation stack...")
+
+    stack_name = get_efs_vol_cfn_stack_name(volume_name)
+
+    # Create the CloudFormation stack
+    capabilities = ["CAPABILITY_NAMED_IAM"]
+    creation_queued, in_created_state = create_cloudformation_stack(
+        template_file_path="configurations/cloudformation/volumes/efs/efs_volume.yaml",
+        stack_name=stack_name,
+        parameters=parameters,
+        capabilities=capabilities)
+    if creation_queued:
+        stream_stack_events(stack_name)
+
+    stack_status = get_stack_status_from_stack_name(stack_name)
+    if stack_status != CfnStackStatus.CREATE_COMPLETE:
+        click.echo(click.style(f"Something went wrong. Please check the CloudFormation stack {stack_name} for details.", fg=CliColors.ERROR.value))
+        return None
+    else:
+        click.echo("EFS volume CloudFormation stack created.")
+        volume_id_key = f"{get_cluster_name()}-EFSFileSystemId-{volume_name}"
+        volume_id = get_cfn_stack_export_values([volume_id_key]).get(volume_id_key, None)
+        return volume_id
+
+def is_existing_efs_storage_class(context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+    k8s_api_client = config.new_client_from_config(context=context_name)
+
+    api_instance = client.StorageV1Api(k8s_api_client)
+    existing_storage_classes = api_instance.list_storage_class().items
+    return any(sc.metadata.name == 'efs-sc' for sc in existing_storage_classes)
+
+
+def create_efs_volume_k8s_resources(volume_name: str, efs_id: str, env: Optional[str] = None) -> bool:
+    chart_path = resource_filename("tensorkube", 'configurations/helm-charts/volumes/efs-volume')
+    namespace = env if env else DEFAULT_NAMESPACE
+    #TODO: use appropriate cluster context
+
+    try:
+        subprocess.run(
+            ["helm", "upgrade", "--install", f"efs-volume-{volume_name}-{namespace}", chart_path, "--create-namespace",
+             "--namespace", namespace, "--set", f"volumeName={volume_name}", "--set", f"namespace={namespace}",
+             "--set", f"fileSystemId={efs_id}"], check=True,
+        )
+        return True
+    except subprocess.CalledProcessError as e:
+        print(f"Failed to create k8s resources for efs volume mount: {e}")
+        return False
+
+
+def check_existing_efs_volume_k8s_resources(volume_name: str, env: Optional[str] = None, context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+
+    namespace = env if env else DEFAULT_NAMESPACE
+
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    api_instance = client.CoreV1Api(k8s_api_client)
+
+    pv_name = get_efs_vol_pv_name(volume_name, namespace)
+    pvc_name = get_efs_vol_pvc_name(volume_name, namespace)
+
+    existing_pvs = api_instance.list_persistent_volume().items
+    existing_pvcs = api_instance.list_namespaced_persistent_volume_claim(namespace=namespace).items
+
+    pv_exists = any(pv.metadata.name == pv_name for pv in existing_pvs)
+    pvc_exists = any(pvc.metadata.name == pvc_name for pvc in existing_pvcs)
+
+    return pv_exists and pvc_exists
+
+
+
 def apply_storage_class(context_name: Optional[str] = None):
     # Load the Kubernetes configuration
     if not context_name:
@@ -495,3 +673,63 @@ def delete_efs_directory_for_deployment(sanitised_project_name: str, env: Option
 
     utils.create_from_dict(k8s_api_client, cleanup_config)
     click.echo('Deployed a delete config job')
+
+
+def list_efs_volumes():
+    efs_client = boto3.client('efs')
+    response = efs_client.describe_file_systems()
+    file_systems = response['FileSystems']
+    efs_volume_list = []
+    for fs in file_systems:
+        tags = efs_client.describe_tags(FileSystemId=fs['FileSystemId'])['Tags']
+        created_by = next((tag['Value'] for tag in tags if tag['Key'] == 'CreatedBy'), None)
+        efs_cluster_name = next((tag['Value'] for tag in tags if tag['Key'] == 'ClusterName'), None)
+        if created_by != 'Tensorfuse' or efs_cluster_name != get_cluster_name():
+            continue
+
+        name = next((tag['Value'] for tag in tags if tag['Key'] == 'Name'), None)
+
+        efs_volume_list.append({"Name": name, "FileSystemId": fs['FileSystemId'], "LifeCycleState": fs['LifeCycleState']})
+
+    return efs_volume_list
+
+
+def delete_efs_volume_cfn(volume_name: str) -> bool:
+    stack_name = get_efs_vol_cfn_stack_name(volume_name)
+    try:
+        delete_generic_cfn_stack(stack_name)
+        return True
+    except Exception as e:
+        click.echo(f"Failed to delete EFS volume CloudFormation stack {stack_name}: {e}")
+        return False
+
+
+def delete_efs_volume_k8s_resources(volume_name: str, context_name: Optional[str] = None):
+    #Get all pvs in all namespaces with prefix f"efs-pv-{volume_name}-"
+
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    api_instance = client.CoreV1Api(k8s_api_client)
+    pvs = api_instance.list_persistent_volume().items
+    helm_deployments = []
+    for pv in pvs:
+        if pv.metadata.name.startswith(f"efs-pv-{volume_name}-"):
+            helm_deployments.append({
+                "namespace": pv.metadata.annotations['meta.helm.sh/release-namespace'],
+                "name": pv.metadata.annotations['meta.helm.sh/release-name']}
+            )
+    for deployment in helm_deployments:
+        try:
+            subprocess.run(
+                ["helm", "delete", deployment['name'], "--namespace", deployment['namespace']], check=True
+            )
+
+        except subprocess.CalledProcessError as e:
+            print(f"Failed to delete helm chart {deployment['name']} in namespace '{deployment['namespace']}': {e}")
+            return False
+
+    return True
