@@ -1,11 +1,12 @@
 import json
 import boto3
+import click
 
 from botocore.exceptions import ClientError
 from pkg_resources import resource_filename
 from typing import Dict
 
-from tensorkube.constants import REGION, get_cluster_name
+from tensorkube.constants import REGION, get_cluster_name, CliColors
 from tensorkube.services.aws_service import get_aws_account_id
 
 
@@ -119,7 +120,7 @@ def create_or_update_iam_role_with_service_account_cluster_access(account_no: st
 
         # Get the current trust policy
         current_trust_policy = response['Role']['AssumeRolePolicyDocument']
-        current_sub_values = current_trust_policy['Statement'][0]['Condition']['StringEquals'].get("{}:sub".format(oidc_issuer), None)
+        current_sub_values = current_trust_policy['Statement'][0]['Condition']['StringEquals'].get("{}:sub".format(oidc_issuer), [])
         # If it's a string, convert it into a list for consistency
         if isinstance(current_sub_values, str):
             current_sub_values = [current_sub_values]
@@ -183,7 +184,23 @@ def delete_role(role_name, region=REGION):
 
 def delete_policy(account_no, policy_name, region=REGION):
     client = boto3.client('iam', region_name=region)
-    response = client.delete_policy(PolicyArn='arn:aws:iam::{}:policy/{}'.format(account_no, policy_name))
+    policy_arn = f'arn:aws:iam::{account_no}:policy/{policy_name}'
+    #delete all policy versions
+    policy = client.get_policy(PolicyArn=policy_arn)
+    default_version = policy['Policy']['DefaultVersionId']
+    # retrieve all policy versions
+    versions = client.list_policy_versions(PolicyArn=policy_arn)
+    for version in versions['Versions']:
+        if version['VersionId'] != default_version:
+            client.delete_policy_version(PolicyArn=policy_arn, VersionId=version['VersionId'])
+    response = client.delete_policy(PolicyArn=policy_arn)
+    return response
+
+
+def update_iam_policy(account_no, policy_name, policy_document, region=REGION):
+    client = boto3.client('iam', region_name=region)
+    response = client.create_policy_version(PolicyArn='arn:aws:iam::{}:policy/{}'.format(account_no, policy_name),
+                                           PolicyDocument=json.dumps(policy_document), SetAsDefault=True)
     return response
 
 
@@ -282,5 +299,101 @@ def create_access_key(user_name, region=REGION):
     return access_key['AccessKeyId'], access_key['SecretAccessKey']
 
 
+def get_policy_source_arn():
+    sts = boto3.client('sts')
+    caller_arn = sts.get_caller_identity()['Arn']
+    if caller_arn.endswith(':root'):
+        return caller_arn
+    elif ":user/" in caller_arn:
+        return caller_arn
+    elif ":assumed-role/" in caller_arn:
+        role_name = caller_arn.split('/')[1]
+        account_id = caller_arn.split(':')[4]
+        return f"arn:aws:iam::{account_id}:role/{role_name}"
+    else:
+        return None
 
 
+def has_simulate_principal_policy_permissions(policy_source_arn: str) -> bool:
+    iam_client = boto3.client('iam')
+    try:
+        test_action = ['iam:SimulatePrincipalPolicy']
+        test_resource = ['*']
+        iam_client.simulate_principal_policy(
+            PolicySourceArn=policy_source_arn,
+            ActionNames=test_action,
+            ResourceArns=test_resource
+        )
+        return True
+    except ClientError as e:
+        if e.response['Error']['Code'] == 'AccessDenied':
+            click.echo(click.style("You do NOT have permissions to call 'iam:SimulatePrincipalPolicy' on Resource '*'.", fg=CliColors.ERROR.value))
+            click.echo(click.style("Tensorkube uses the 'iam:SimulatePrincipalPolicy' to check for all other permissions required for running the configure command. You can refer to our docs for the full list of permissions we check for, here: https://tensorfuse.io/docs/reference/permissions/configure_permissions", fg=CliColors.WARNING.value))
+            click.echo(click.style("Please request the necessary IAM permissions before running this check.", fg=CliColors.ERROR.value))
+            return False
+        else:
+            raise
+
+
+def has_configure_permissions() -> bool:
+    click.echo("Checking if you have the necessary AWS permissions to run the configure command...")
+
+    policy_source_arn = get_policy_source_arn()
+    if not policy_source_arn:
+        raise Exception("Could not determine the policy source ARN.")
+
+    elif  policy_source_arn.endswith(':root'):
+        click.echo(click.style("You are running this as the root user.", fg=CliColors.WARNING.value))
+        click.echo(click.style("Skipping 'SimulatePrincipalPolicy' check for root ARN as it is not possible to run this check for the root user.", fg=CliColors.WARNING.value))
+        click.echo(click.style("Please ensure no AWS Organizations Service Control Policies limit your root account.", fg=CliColors.WARNING.value))
+        click.echo(click.style("You can refer to our docs for the full list of permissions we check for, here: https://tensorfuse.io/docs/reference/permissions/configure_permissions", fg=CliColors.WARNING.value))
+        return True
+
+
+    click.echo(f"Running tests for: {policy_source_arn}")
+    if not has_simulate_principal_policy_permissions(policy_source_arn):
+        return False
+
+
+    policy_file_path = resource_filename('tensorkube', 'permissions/configure_permissions.json')
+    with open(policy_file_path, 'r') as f:
+        policy_doc = json.load(f)
+
+    statements = policy_doc.get("Statement", [])
+    if not isinstance(statements, list):
+        # Some policies have Statement as dict if there's only one.
+        statements = [statements]
+
+    iam_client = boto3.client('iam')
+    account_no = get_aws_account_id()
+    all_allowed = True
+    for idx, statement in enumerate(statements):
+        actions = statement.get("Action", [])
+        resources = statement.get("Resource", [])
+
+        if isinstance(actions, str):
+            actions = [actions]
+        if isinstance(resources, str):
+            resources = [resources]
+        resources = [resource.replace('<ACCOUNT_NO>', account_no) for resource in resources]
+
+        try:
+            response = iam_client.simulate_principal_policy(
+                PolicySourceArn=policy_source_arn,
+                ActionNames=actions,
+                ResourceArns=resources
+            )
+        except ClientError as e:
+            print(f"Error simulating statement: {e}")
+            continue
+
+        eval_results = response.get('EvaluationResults', [])
+        for result in eval_results:
+            action_name = result.get('EvalActionName')
+            resource_name = result.get('EvalResourceName')
+            decision = result.get('EvalDecision')
+            if decision != 'allowed':
+                click.echo(click.style(f"Permissions Missing. Action: {action_name}, Resource: {resource_name} => Decision: {decision}", fg=CliColors.ERROR.value))
+                all_allowed = False
+
+    return all_allowed
