@@ -5,24 +5,30 @@ import boto3
 import click
 import yaml
 from kubernetes import config, client
+from kubernetes.client import ApiException
 from pkg_resources import resource_filename
 from typing import Optional, List
 
-from tensorkube.constants import REGION, get_cluster_name, CliColors
+from tensorkube.constants import REGION, get_cluster_name, CliColors, get_job_sidecar_iam_role_name, \
+    GPU_DEPLOYMENT_MIN_MEMORY, GPU_DEPLOYMENT_MIN_CPU
 from tensorkube.helpers import sanitise_name, extract_workdir_from_dockerfile, extract_command_from_dockerfile
 from tensorkube.services.aws_service import get_aws_account_id, get_bucket_name, get_credentials
-from tensorkube.services.dynamodb_service import get_dynamodb_table_name
+from tensorkube.services.dynamodb_service import get_dynamodb_table_name, get_dynamodb_access_policy_name
 from tensorkube.services.eks_service import get_cluster_oidc_issuer_url, install_keda, delete_keda_from_cluster
 from tensorkube.services.environment_service import delete_environment, create_new_environment
 from tensorkube.services.filesystem_service import delete_efs_directory_for_deployment
 from tensorkube.services.iam_service import create_sqs_access_policy, create_sqs_access_role, attach_role_policy, \
-    get_role_name_for_prefix, create_dynamo_access_policy, delete_policy, detach_role_policy, delete_iam_role
+    get_role_name_for_prefix, create_dynamo_access_policy, delete_policy, detach_role_policy, delete_iam_role, \
+    create_or_update_iam_role_with_service_account_cluster_access
 from tensorkube.services.k8s_service import get_tensorkube_cluster_context_name, get_efs_claim_name, \
     list_keda_scaled_jobs, list_trigger_authentications, delete_trigger_authentication, create_service_account, \
-    create_k8s_role, create_role_binding, create_aws_secret, is_existing_scaled_job, generate_kubeconfig
+    create_k8s_role, create_role_binding, create_aws_secret, is_existing_scaled_job, generate_kubeconfig, \
+    patch_service_account
 from tensorkube.services.knative_service import get_instance_family_from_gpu_type, get_supported_gpu_families
+from tensorkube.services.s3_access_service import get_s3_access_policy_name
 from tensorkube.services.s3_service import create_s3_bucket
-from tensorkube.services.sqs_service import create_sqs_queue, queue_message, delete_sqs_queue
+from tensorkube.services.sqs_service import create_sqs_queue, queue_message, delete_sqs_queue, \
+    get_sqs_access_policy_name
 from tensorkube.services.nydus import get_nydus_runtime_class_name
 from tensorkube.services.nodepool import get_gpu_nodepool
 
@@ -32,16 +38,18 @@ def create_cloud_resources_for_queued_job_support():
     cluster_name = get_cluster_name()
     oidc_issuer_url = get_cluster_oidc_issuer_url(cluster_name)
 
-    sqs_policy_name = f"{cluster_name}-sqs-access-policy"
+    sqs_policy_name = get_sqs_access_policy_name()
+    s3_access_policy_name = get_s3_access_policy_name()
     role_name = f"{cluster_name}-sqs-access-role"
     policy = create_sqs_access_policy(sqs_policy_name)
-    dyanmo_policy_name = f"{cluster_name}-dynamo-access-policy"
+    dyanmo_policy_name = get_dynamodb_access_policy_name()
     create_dynamo_access_policy(dyanmo_policy_name)
     click.echo("Policy created")
     role = create_sqs_access_role(get_aws_account_id(), oidc_issuer_url, role_name, 'keda', 'keda-operator')
     click.echo("Role created")
     attach_role_policy(get_aws_account_id(), sqs_policy_name, role_name)
     attach_role_policy(get_aws_account_id(), dyanmo_policy_name, role_name)
+    attach_role_policy(account_no=get_aws_account_id(), policy_name=s3_access_policy_name, role_name=role_name)
     click.echo("Policy attached to role")
 
     # TODO!: on Nydus implementation, create new role and service account for combined access to SQS and DynamoDB to be used in ScaledJob
@@ -51,6 +59,8 @@ def create_cloud_resources_for_queued_job_support():
     attach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=karpenter_role)
     attach_role_policy(account_no=get_aws_account_id(), policy_name=dyanmo_policy_name, role_name=eksctl_role)
     attach_role_policy(account_no=get_aws_account_id(), policy_name=dyanmo_policy_name, role_name=karpenter_role)
+    attach_role_policy(account_no=get_aws_account_id(), policy_name=s3_access_policy_name, role_name=eksctl_role)
+    attach_role_policy(account_no=get_aws_account_id(), policy_name=s3_access_policy_name, role_name=karpenter_role)
     click.echo("SQS access policy attached to nodes")
 
     create_new_environment('keda')
@@ -85,8 +95,16 @@ def create_trigger_authentication_for_aws_sqs(sqs_access_iam_role_arn: str, cont
         trigger_auth_json = yaml.safe_load(f)
     trigger_auth_json['spec']['podIdentity']['roleArn'] = sqs_access_iam_role_arn
 
-    k8s_client.create_namespaced_custom_object('keda.sh', 'v1alpha1', 'keda', 'triggerauthentications',
-                                               trigger_auth_json)
+    try:
+        k8s_client.create_namespaced_custom_object('keda.sh', 'v1alpha1', 'keda', 'triggerauthentications',
+                                                 trigger_auth_json)
+    except ApiException as e:
+        if e.status == 409:
+            click.echo("Trigger authentication already exists. Skipping creation.")
+        else:
+            print(f"Error while creating trigger authentication: {e}")
+            raise e
+
 
 
 def get_queue_name_for_job(job_name: str):
@@ -136,8 +154,9 @@ def deploy_job(job_name: str, gpus: int, gpu_type: Optional[str], cpu: int, memo
         bucket_name = get_bucket_name(env_name=env, type='train')
         scaled_job_json['metadata']['name'] = job_name
         scaled_job_json['spec']['triggers'][0]['metadata']['queueURL'] = job_queue_url
-        scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['env'].extend([
-            {'name': 'AWS_REGION', 'value': region}, {'name': 'QUEUE_URL', 'value': job_queue_url}, {'name': 'JOB_NAME', 'value': job_name}, {'name': 'LORA_ADAPTER_BUCKET', 'value': bucket_name}])
+        scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['env'] = [
+            {'name': 'AWS_REGION', 'value': region}, {'name': 'QUEUE_URL', 'value': job_queue_url},
+            {'name': 'JOB_NAME', 'value': job_name}, {'name': 'LORA_ADAPTER_BUCKET', 'value': bucket_name}]
         scaled_job_json['spec']['triggers'][0]['metadata']['region'] = region
         scaled_job_json['spec']['maxReplicaCount'] = max_scale
 
@@ -172,12 +191,20 @@ def deploy_job(job_name: str, gpus: int, gpu_type: Optional[str], cpu: int, memo
         scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['resources'] = {'requests': {},
                                                                                                      'limits': {}}
         if gpus > 0:
-            if memory > 2000:
-                scaled_job_json['spec']['template']['spec']['containers'][0]['resources']['requests'][
+            if memory >= GPU_DEPLOYMENT_MIN_MEMORY:
+                scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['resources']['requests'][
                     'memory'] = f'{str(int(memory))}M'
-            if cpu > 2000:
-                scaled_job_json['spec']['template']['spec']['containers'][0]['resources']['requests'][
+            else:
+                click.echo(click.style(f"Memory is not specified or is less than the minimum required for GPU deployment. Setting to minimum: {GPU_DEPLOYMENT_MIN_MEMORY}M.", fg=CliColors.WARNING.value))
+                scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['resources']['requests'][
+                    'memory'] = f'{str(GPU_DEPLOYMENT_MIN_MEMORY)}M'
+            if cpu >= GPU_DEPLOYMENT_MIN_CPU:
+                scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['resources']['requests'][
                     'cpu'] = f'{str(int(cpu))}m'
+            else:
+                click.echo(click.style(f"CPU is not specified or is less than the minimum required for GPU deployment. Setting to minimum: {GPU_DEPLOYMENT_MIN_CPU}m.", fg=CliColors.WARNING.value))
+                scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['resources']['requests'][
+                    'cpu'] = f'{str(GPU_DEPLOYMENT_MIN_CPU)}m'
     
             scaled_job_json['spec']['jobTargetRef']['template']['spec']['containers'][0]['resources']['requests'][
                 'nvidia.com/gpu'] = gpus
@@ -304,9 +331,9 @@ def deploy_job(job_name: str, gpus: int, gpu_type: Optional[str], cpu: int, memo
             if gpus > 0:
                 scaled_job_json['spec']['jobTargetRef']['template']['spec']['runtimeClassName'] = get_nydus_runtime_class_name()
                 if 'nodeSelector' in scaled_job_json['spec']['jobTargetRef']['template']['spec']:
-                    scaled_job_json['spec']['jobTargetRef']['template']['spec']['nodeSelector']['nodepool'] = get_gpu_nodepool()
+                    scaled_job_json['spec']['jobTargetRef']['template']['spec']['nodeSelector']['karpenter.sh/nodepool'] = get_gpu_nodepool()
                 else:
-                    scaled_job_json['spec']['jobTargetRef']['template']['spec']['nodeSelector'] = {'nodepool': get_gpu_nodepool()}
+                    scaled_job_json['spec']['jobTargetRef']['template']['spec']['nodeSelector'] = {'karpenter.sh/nodepool': get_gpu_nodepool()}
             
     
     scaled_job_json['spec']['jobTargetRef']['template']['spec']['initContainers'][0]['command'] = ["/bin/sh", "-c", f"""\
@@ -464,16 +491,27 @@ def delete_dynamo_table():
 
 
 def teardown_job_queue_support():
-    jobs = list_keda_scaled_jobs()
-    for job in jobs['items']:
-        job_name = job['metadata']['name']
-        delete_all_job_resources(job_name)
+    try:
+        jobs = list_keda_scaled_jobs()
+        for job in jobs['items']:
+            job_name = job['metadata']['name']
+            delete_all_job_resources(job_name)
+    except Exception as e:
+        print(f"Error deleting jobs: {e}")
+
+    try:
+        trigger_auths = list_trigger_authentications()
+        for trigger_auth in trigger_auths['items']:
+            trigger_auth_name = trigger_auth['metadata']['name']
+            delete_trigger_authentication(trigger_auth_name)
+    except Exception as e:
+        print(f"Error deleting trigger authentications: {e}")
+
+    try:
+        delete_dynamo_table()
+    except Exception as e:
+        print(f"Error deleting DynamoDB table: {e}")
 
-    trigger_auths = list_trigger_authentications()
-    for trigger_auth in trigger_auths['items']:
-        trigger_auth_name = trigger_auth['metadata']['name']
-        delete_trigger_authentication(trigger_auth_name)
-    delete_dynamo_table()
     cluster_name = get_cluster_name()
     sqs_policy_name = f"{cluster_name}-sqs-access-policy"
     sqs_role_name = f"{cluster_name}-sqs-access-role"
@@ -482,19 +520,64 @@ def teardown_job_queue_support():
     eksctl_role = get_role_name_for_prefix(prefix=f"eksctl-{get_cluster_name()}-nodegroup-")
     karpenter_role = get_role_name_for_prefix(prefix=f"KarpenterNodeRole-{get_cluster_name()}")
 
-    detach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=eksctl_role)
-    detach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=karpenter_role)
-    detach_role_policy(account_no=get_aws_account_id(), policy_name=dynamo_policy_name, role_name=eksctl_role)
-    detach_role_policy(account_no=get_aws_account_id(), policy_name=dynamo_policy_name, role_name=karpenter_role)
+    try:
+        detach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=eksctl_role)
+    except Exception as e:
+        print(f"Error detaching policy from role: {e}")
 
-    detach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=sqs_role_name)
-    delete_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name)
-    delete_iam_role(role_name=sqs_role_name)
+    try:
+        detach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=karpenter_role)
+    except Exception as e:
+        print(f"Error detaching policy from role: {e}")
 
-    delete_policy(account_no=get_aws_account_id(), policy_name=dynamo_policy_name)
+    try:
+        detach_role_policy(account_no=get_aws_account_id(), policy_name=dynamo_policy_name, role_name=eksctl_role)
+    except Exception as e:
+        print(f"Error detaching policy from role: {e}")
 
-    delete_keda_from_cluster()
-    delete_environment('keda')
+    try:
+        detach_role_policy(account_no=get_aws_account_id(), policy_name=dynamo_policy_name, role_name=karpenter_role)
+    except Exception as e:
+        print(f"Error detaching policy from role: {e}")
+
+    try:
+        detach_role_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name, role_name=sqs_role_name)
+    except Exception as e:
+        print(f"Error detaching policy from role: {e}")
+
+    try:
+        delete_iam_role(role_name=sqs_role_name)
+    except Exception as e:
+        print(f"Error deleting role: {e}")
+    try:
+        delete_iam_role(role_name=get_job_sidecar_iam_role_name())
+    except Exception as e:
+        print(f"Error deleting role: {e}")
+
+    try:
+        delete_iam_role(role_name='job-queue-sidecar-keda-role')
+    except Exception as e:
+        print(f"Error deleting role: {e}")
+
+    try:
+        delete_policy(account_no=get_aws_account_id(), policy_name=sqs_policy_name)
+    except Exception as e:
+        print(f"Error deleting policy: {e}")
+
+    try:
+        delete_policy(account_no=get_aws_account_id(), policy_name=dynamo_policy_name)
+    except Exception as e:
+        print(f"Error deleting policy: {e}")
+
+    try:
+        delete_keda_from_cluster()
+    except Exception as e:
+        print(f"Error deleting Keda: {e}")
+
+    try:
+        delete_environment('keda')
+    except Exception as e:
+        print(f"Error deleting environment: {e}")
 
 
 # EXPOSE
@@ -569,15 +652,28 @@ def get_job_status(job_name: str, job_id: str):
 def get_job_sidecar_service_account_name(namespace: Optional[str] = None):
     if not namespace:
         namespace = "keda"
-    return f"job-queue-sidecar-{namespace}-sa"
+    return f"job-queue-sidecar-{namespace}-sa-v2"
+
+
+def get_job_sidecar_k8s_role_name(namespace: Optional[str] = None):
+    if not namespace:
+        namespace = "keda"
+    return f"job-queue-sidecar-{namespace}-role"
+
+
+def get_job_sidecar_rolebinding_name(namespace: Optional[str] = None):
+    if not namespace:
+        namespace = "keda"
+    return f"job-queue-sidecar-{namespace}-rb-v2"
 
 
 def create_sa_role_rb_for_job_sidecar(namespace: Optional[str] = None):
     if not namespace:
         namespace = "keda"
     sa_name = get_job_sidecar_service_account_name(namespace)
-    role_name = f"job-queue-sidecar-{namespace}-role"
-    rb_name = f"job-queue-sidecar-{namespace}-rb"
+    role_name = get_job_sidecar_k8s_role_name(namespace)
+    rb_name = get_job_sidecar_rolebinding_name(namespace)
+    iam_role_name = get_job_sidecar_iam_role_name()
 
     pod_tracking_rules = [{"apiGroups": [""], "resources": ["pods"], "verbs": ["get", "list"]}]
 
@@ -587,4 +683,17 @@ def create_sa_role_rb_for_job_sidecar(namespace: Optional[str] = None):
     click.echo("Role created")
     create_role_binding(rb_name=rb_name, role_name=role_name, sa_name=sa_name, rb_namespace=namespace,
                         sa_namespace=namespace)
+
+    iam_role = create_or_update_iam_role_with_service_account_cluster_access(get_aws_account_id(),
+                                                                         get_cluster_oidc_issuer_url(
+                                                                             get_cluster_name()),
+                                                                         iam_role_name,
+                                                                         get_job_sidecar_service_account_name(),
+                                                                         'keda')
+    iam_role_arn = iam_role['Role']['Arn']
+    attach_role_policy(account_no=get_aws_account_id(), policy_name=get_s3_access_policy_name(), role_name=iam_role_name)
+    attach_role_policy(account_no=get_aws_account_id(), policy_name=get_dynamodb_access_policy_name(), role_name=iam_role_name)
+    attach_role_policy(account_no=get_aws_account_id(), policy_name=get_sqs_access_policy_name(), role_name=iam_role_name)
+    patch_service_account(iam_role_arn, get_job_sidecar_service_account_name(), "keda")
+
     click.echo("Role Binding created")
