@@ -1,8 +1,10 @@
+import os
 import subprocess
 import time
-from threading import Thread
+from threading import Thread, Event
 from typing import List, Optional, Dict, Any
 
+import boto3
 import click
 import yaml
 from kubernetes import client, config, utils, watch
@@ -12,23 +14,22 @@ from pkg_resources import resource_filename
 from tensorkube.constants import NAMESPACE, REGION, PodStatus, BUILD_TOOL, DEFAULT_NAMESPACE, TENSORFUSE_NAMESPACES, \
     get_cluster_name, CliColors
 from tensorkube.services.aws_service import get_aws_account_id
-import boto3
-import os
+
 
 def get_s3_pv_name(env_name: Optional[str] = None):
-    if env_name:
+    if env_name and env_name != DEFAULT_NAMESPACE:
         return f"s3-pv-env-{env_name}"
     return "s3-pv"
 
 
 def get_s3_claim_name(env_name: Optional[str] = None):
-    if env_name:
+    if env_name and env_name != DEFAULT_NAMESPACE:
         return f"s3-claim-env-{env_name}"
     return "s3-claim"
 
 
 def get_efs_claim_name(env_name: Optional[str] = None):
-    if env_name:
+    if env_name and env_name != DEFAULT_NAMESPACE:
         return f"efs-pvc-env-{env_name}"
     return "efs-pvc"
 
@@ -168,11 +169,6 @@ def create_build_pv_and_pvc(bucket_name: str, region: str = REGION, env: Optiona
             raise e
 
 
-        
-
-
-
-
 def apply_image_cleanup_job(sanitised_project_name: str, image_tags: List[str], env: Optional[str] = None,
                             context_name: Optional[str] = None):
     if not context_name:
@@ -284,7 +280,7 @@ def delete_all_jobs_in_namespace(namespace: str, context_name: Optional[str] = N
 
 
 def start_streaming_pod(pod_name, namespace, status=None, container_name=None, retry_number: int = 0,
-                        max_retries: int = 4, context_name: Optional[str] = None):
+                        max_retries: int = 4, context_name: Optional[str] = None, stop_event: Optional[Event] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
@@ -300,6 +296,9 @@ def start_streaming_pod(pod_name, namespace, status=None, container_name=None, r
         events_streamed_upto_index = 0
 
         while True:
+            if stop_event and stop_event.is_set():
+                print("Event streaming stopped.")
+                return
             pod = v1.read_namespaced_pod(name=pod_name, namespace=namespace)
             events = v1.list_namespaced_event(namespace=namespace, field_selector=f'involvedObject.name={pod_name}')
             for event in events.items[events_streamed_upto_index:]:
@@ -318,8 +317,23 @@ def start_streaming_pod(pod_name, namespace, status=None, container_name=None, r
     try:
         last_log_printed = None
         while True:
-            logs = v1.read_namespaced_pod_log(name=pod_name, namespace=namespace, container=container_name,
-                                              since_seconds=5)
+            if stop_event and stop_event.is_set():
+                print("Stop event received, stopping log streaming.")
+                return
+            try:
+                logs = v1.read_namespaced_pod_log(name=pod_name, namespace=namespace, container=container_name,
+                                                  since_seconds=5)
+            except UnicodeDecodeError:
+                print(f'Got invalid characters in log. trying to process.')
+                # If automatic decoding fails, fetch raw bytes and decode with error replacement.
+                response = v1.read_namespaced_pod_log(
+                    name=pod_name,
+                    namespace=namespace,
+                    container=container_name,
+                    since_seconds=5,
+                    _preload_content=False
+                )
+                logs = response.data.decode('utf-8', errors='replace')
             logs_by_line = logs.split('\n')
             logs_by_line = logs_by_line[:len(logs_by_line) - 1]
             if logs_by_line:
@@ -390,6 +404,61 @@ def get_pod_status_from_job(job_name: str, namespace: str = DEFAULT_NAMESPACE, c
         else:
             raise e
 
+def set_stop_event_post_ksvc_ready(service_name, namespace, stop_event, context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    custom_api = client.CustomObjectsApi(k8s_api_client)
+    v1 = client.CoreV1Api(k8s_api_client)
+
+    group = "serving.knative.dev"
+    version = "v1"
+    plural = "services"
+
+    max_wait_seconds = 600  # maximum time to wait (adjust as needed)
+    start_time = time.time()
+
+    while not stop_event.is_set():
+        try:
+            ksvc = custom_api.get_namespaced_custom_object(
+                group=group,
+                version=version,
+                namespace=namespace,
+                plural=plural,
+                name=service_name
+            )
+            conditions = ksvc.get("status", {}).get("conditions", [])
+            for condition in conditions:
+                if condition.get("type") == "Ready" and condition.get("status") == "True":
+                    print(f"Service {service_name} is ready.")
+                    stop_event.set()
+                    return
+        except client.ApiException as e:
+            if e.status == 404:
+                print(f"Service {service_name} not found in namespace {namespace}")
+            else:
+                raise
+        # Check pods for CrashLoopBackOff status
+        pods = v1.list_namespaced_pod(
+            namespace=namespace,
+            label_selector=f'serving.knative.dev/service={service_name}'
+        )
+        for pod in pods.items:
+            for cs in pod.status.container_statuses or []:
+                waiting_state = cs.state.waiting
+                if waiting_state and waiting_state.reason == "CrashLoopBackOff":
+                    click.echo(click.style(f"Pod {pod.metadata.name} is in CrashLoopBackOff state.", fg='red', bold=True))
+                    stop_event.set()
+                    return
+        # Check if we've exceeded the maximum wait time
+        if (time.time() - start_time) > max_wait_seconds:
+            click.echo(click.style(f"Service {service_name} did not become ready within {max_wait_seconds} seconds. Exiting.", fg='red', bold=True))
+            stop_event.set()
+            return
+        time.sleep(5)
 
 def start_streaming_service(service_name, namespace, context_name: Optional[str] = None):
     # Load kube config
@@ -404,17 +473,30 @@ def start_streaming_service(service_name, namespace, context_name: Optional[str]
 
     # Stream the service status
     try:
-        while True:
+        pod_name = None
+        while pod_name is None:
             pods = v1.list_namespaced_pod(namespace, label_selector=f'serving.knative.dev/service={service_name}')
             if pods.items:
                 sorted_pods = sorted(pods.items, key=lambda x: x.metadata.name, reverse=True)
-                print(f"Pods scheduled for service {service_name}: {sorted_pods[0].metadata.name}")
-                break
-            time.sleep(5)
+                pod_name = sorted_pods[0].metadata.name
+                print(f"Found pod {pod_name} for service {service_name}.")
+            else:
+                time.sleep(5)
+
+        # Create an Event to signal the log streaming thread when the service is ready.
+        stop_event = Event()
         # Start streaming the logs from the pod
-        thread = Thread(target=start_streaming_pod,
-                        args=(sorted_pods[0].metadata.name, namespace, PodStatus.RUNNING, 'user-container'))
-        thread.start()
+        streaming_thread = Thread(target=start_streaming_pod,
+                        args=(pod_name, namespace, None, 'user-container'),
+                        kwargs={'stop_event': stop_event, 'context_name': context_name})
+        streaming_thread.start()
+        set_stop_event_post_ksvc_ready(
+            service_name=service_name,
+            namespace=namespace,
+            stop_event=stop_event,
+            context_name=context_name
+        )
+        streaming_thread.join()  # Ensure the streaming thread is properly joined.
     except client.ApiException as e:
         if e.status == 404:
             print(f"Service {service_name} not found in namespace {namespace}")
@@ -697,7 +779,7 @@ def get_image_tags_to_retain(sanitised_project_name: str, service_name: str, nam
 
 
 def get_pod_name_corresponing_to_job(job_name: str, namespace: str = DEFAULT_NAMESPACE,
-                                     context_name: Optional[str] = None, max_retries = 5) -> Optional[str]:
+                                     context_name: Optional[str] = None, max_retries=5) -> Optional[str]:
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
@@ -760,11 +842,12 @@ def ssh_into_pod(pod_name: str, namespace: str, context_name: Optional[str] = No
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
             return None
-    
+
     command = ["kubectl", "--context", f"{context_name}", "exec", "-it", pod_name, "-n", namespace, "-c",
                "user-container", "--", "sh"]
     subprocess.run(command)
 
+
 def ssh_into_pod_with_podman(pod_name: str, namespace: str, context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
@@ -813,49 +896,51 @@ def remove_invalid_contexts(tensorkube_contexts: List[str]) -> Optional[str]:
     return None
 
 
-def create_secret(name: str, namespace: str, data: Dict[str, str], force: bool = False, context_name: Optional[str] = None):
+def create_secret(name: str, namespace: str, data: Dict[str, str], force: bool = False,
+                  context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
             raise Exception("No context found for the cluster. Please configure Tensorfuse properly.")
     k8s_api_client = config.new_client_from_config(context=context_name)
     v1 = client.CoreV1Api(k8s_api_client)
- 
+
     secret = client.V1Secret()
     secret.metadata = client.V1ObjectMeta(name=name, namespace=namespace)
     secret.string_data = data
 
-
     try:
         # Check if the secret already exists
         v1.read_namespaced_secret(name, namespace)
         if force:
             # If the secret exists and force is True, update the secret
             v1.replace_namespaced_secret(name, namespace, body=secret)
-            click.echo(click.style("Updated already existing secret ", fg=CliColors.SUCCESS.value) 
-                       + click.style(name, bold=True, fg=CliColors.SUCCESS.value)
-                       + click.style(" in namespace ", fg=CliColors.SUCCESS.value) 
-                       + click.style(namespace, bold=True, fg=CliColors.SUCCESS.value))
+            click.echo(click.style("Updated already existing secret ", fg=CliColors.SUCCESS.value) + click.style(name,
+                                                                                                                 bold=True,
+                                                                                                                 fg=CliColors.SUCCESS.value) + click.style(
+                " in namespace ", fg=CliColors.SUCCESS.value) + click.style(namespace, bold=True,
+                                                                            fg=CliColors.SUCCESS.value))
         else:
-            click.echo(click.style("Secret ", fg=CliColors.WARNING.value) 
-                       + click.style(name,  bold=True, fg=CliColors.WARNING.value)
-                       + click.style(" already exists in namespace ", fg=CliColors.WARNING.value)
-                       + click.style(namespace, bold=True, fg=CliColors.WARNING.value)
-                       + click.style(". Use --force to update the secret.", fg=CliColors.WARNING.value))
+            click.echo(click.style("Secret ", fg=CliColors.WARNING.value) + click.style(name, bold=True,
+                                                                                        fg=CliColors.WARNING.value) + click.style(
+                " already exists in namespace ", fg=CliColors.WARNING.value) + click.style(namespace, bold=True,
+                                                                                           fg=CliColors.WARNING.value) + click.style(
+                ". Use --force to update the secret.", fg=CliColors.WARNING.value))
     except ApiException as e:
         if e.status == 404:
             # If the secret does not exist, create it
             try:
                 v1.create_namespaced_secret(namespace=namespace, body=secret)
-                click.echo(click.style("Secret ", fg=CliColors.SUCCESS.value) 
-               + click.style(name, bold=True, fg=CliColors.SUCCESS.value)
-               + click.style(" created successfully.", fg=CliColors.SUCCESS.value))
+                click.echo(click.style("Secret ", fg=CliColors.SUCCESS.value) + click.style(name, bold=True,
+                                                                                            fg=CliColors.SUCCESS.value) + click.style(
+                    " created successfully.", fg=CliColors.SUCCESS.value))
             except ApiException as e:
                 click.echo(click.style("An error occurred while creating the secret", fg=CliColors.ERROR.value))
                 click.echo(click.style(f"Error: {e}", fg=CliColors.ERROR.value))
         else:
             raise e
 
+
 def list_secrets(namespace: str, context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
@@ -878,26 +963,27 @@ def delete_secret(name: str, namespace: str, context_name: Optional[str] = None)
     v1 = client.CoreV1Api(k8s_api_client)
     try:
         v1.delete_namespaced_secret(name, namespace)
-        click.echo(click.style("Secret ", fg=CliColors.SUCCESS.value) 
-                   + click.style(name, bold=True, fg=CliColors.SUCCESS.value)
-                   + click.style(" deleted successfully.", fg=CliColors.SUCCESS.value))
+        click.echo(click.style("Secret ", fg=CliColors.SUCCESS.value) + click.style(name, bold=True,
+                                                                                    fg=CliColors.SUCCESS.value) + click.style(
+            " deleted successfully.", fg=CliColors.SUCCESS.value))
     except ApiException as e:
         if e.status == 404:
-            click.echo(click.style("Secret ", fg=CliColors.WARNING.value) 
-                       + click.style(name, bold=True, fg=CliColors.WARNING.value)
-                       + click.style(" not found in namespace ", fg=CliColors.WARNING.value) 
-                       + click.style(namespace, bold=True, fg=CliColors.WARNING.value))
+            click.echo(click.style("Secret ", fg=CliColors.WARNING.value) + click.style(name, bold=True,
+                                                                                        fg=CliColors.WARNING.value) + click.style(
+                " not found in namespace ", fg=CliColors.WARNING.value) + click.style(namespace, bold=True,
+                                                                                      fg=CliColors.WARNING.value))
         else:
             raise e
 
 
-def create_configmap(name: str, namespace: str, data: Dict[str, str], force: bool = False, context_name: Optional[str] = None):
+def create_configmap(name: str, namespace: str, data: Dict[str, str], force: bool = False,
+                     context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
             raise Exception("No context found for the cluster. Please configure Tensorfuse properly.")
     k8s_api_client = config.new_client_from_config(context=context_name)
-    v1 = client.CoreV1Api(k8s_api_client) 
+    v1 = client.CoreV1Api(k8s_api_client)
     configmap = client.V1ConfigMap()
     configmap.metadata = client.V1ObjectMeta(name=name, namespace=namespace)
     configmap.data = {"config": yaml.dump(data)}
@@ -917,6 +1003,7 @@ def create_configmap(name: str, namespace: str, data: Dict[str, str], force: boo
         else:
             raise e
 
+
 def get_pods_for_jobs(job_name: str, namespace: str = "default"):
     context_name = get_tensorkube_cluster_context_name()
     if not context_name:
@@ -924,17 +1011,13 @@ def get_pods_for_jobs(job_name: str, namespace: str = "default"):
     k8s_api_client = config.new_client_from_config(context=context_name)
     api = client.CoreV1Api(k8s_api_client)
     try:
-        return api.list_namespaced_pod(
-            namespace=namespace,
-            label_selector=f"job-name={job_name}"
-        )
+        return api.list_namespaced_pod(namespace=namespace, label_selector=f"job-name={job_name}")
     except ApiException as e:
         print(f"Exception when calling CoreV1Api->list_namespaced_pod: {e}")
         return None
 
 
-def list_jobs(namespace: Optional[str] = None, all: bool = False,
-              job_name_prefix: Optional[str] = None):
+def list_jobs(namespace: Optional[str] = None, all: bool = False, job_name_prefix: Optional[str] = None):
     """
     Lists Kubernetes jobs in a namespace or across all namespaces with optional job name prefix filtering
 
@@ -963,10 +1046,7 @@ def list_jobs(namespace: Optional[str] = None, all: bool = False,
 
         # Filter jobs by prefix if specified
         if job_name_prefix:
-            filtered_items = [
-                job for job in job_list.items
-                if job.metadata.name.startswith(job_name_prefix)
-            ]
+            filtered_items = [job for job in job_list.items if job.metadata.name.startswith(job_name_prefix)]
             job_list.items = filtered_items
 
         return job_list
@@ -1003,28 +1083,16 @@ def delete_job(job_name: str, namespace: str = "default") -> bool:
 
         # Delete associated pods first
         try:
-            pods = core_v1.list_namespaced_pod(
-                namespace=namespace,
-                label_selector=f"job-name={job_name}"
-            )
+            pods = core_v1.list_namespaced_pod(namespace=namespace, label_selector=f"job-name={job_name}")
             for pod in pods.items:
-                core_v1.delete_namespaced_pod(
-                    name=pod.metadata.name,
-                    namespace=namespace
-                )
+                core_v1.delete_namespaced_pod(name=pod.metadata.name, namespace=namespace)
         except ApiException as e:
             print(f"Error deleting pods for job {job_name}: {e}")
 
         # Delete the job
         try:
-            batch_v1.delete_namespaced_job(
-                name=job_name,
-                namespace=namespace,
-                body=client.V1DeleteOptions(
-                    propagation_policy='Foreground',
-                    grace_period_seconds=0
-                )
-            )
+            batch_v1.delete_namespaced_job(name=job_name, namespace=namespace,
+                body=client.V1DeleteOptions(propagation_policy='Foreground', grace_period_seconds=0))
             click.echo(click.style(f"Successfully deleted job {job_name}", fg='green'))
             return True
         except ApiException as e:
@@ -1046,12 +1114,8 @@ def list_keda_scaled_jobs():
     k8s_client = client.CustomObjectsApi(api_client)
 
     try:
-        keda_scaled_jobs = k8s_client.list_namespaced_custom_object(
-            group="keda.sh",
-            version="v1alpha1",
-            plural="scaledjobs",
-            namespace="keda"
-        )
+        keda_scaled_jobs = k8s_client.list_namespaced_custom_object(group="keda.sh", version="v1alpha1",
+            plural="scaledjobs", namespace="keda")
         return keda_scaled_jobs
     except ApiException as e:
         print(f"Exception when calling CustomObjectsApi: {e}")
@@ -1067,12 +1131,8 @@ def list_trigger_authentications():
     k8s_client = client.CustomObjectsApi(api_client)
 
     try:
-        trigger_authentications = k8s_client.list_namespaced_custom_object(
-            group="keda.sh",
-            version="v1alpha1",
-            plural="triggerauthentications",
-            namespace="keda"
-        )
+        trigger_authentications = k8s_client.list_namespaced_custom_object(group="keda.sh", version="v1alpha1",
+            plural="triggerauthentications", namespace="keda")
         return trigger_authentications
     except ApiException as e:
         print(f"Exception when calling CustomObjectsApi: {e}")
@@ -1088,17 +1148,13 @@ def delete_trigger_authentication(trigger_auth_name: str):
     k8s_client = client.CustomObjectsApi(api_client)
 
     try:
-        k8s_client.delete_namespaced_custom_object(
-            group="keda.sh",
-            version="v1alpha1",
-            plural="triggerauthentications",
-            namespace="keda",
-            name=trigger_auth_name
-        )
+        k8s_client.delete_namespaced_custom_object(group="keda.sh", version="v1alpha1", plural="triggerauthentications",
+            namespace="keda", name=trigger_auth_name)
     except ApiException as e:
         print(f"Exception when calling CustomObjectsApi: {e}")
         return None
 
+
 def add_user_to_aws_auth_group(user_name, group_name):
     v1 = client.CoreV1Api()
     # details of the aws-auth ConfigMap
@@ -1108,11 +1164,8 @@ def add_user_to_aws_auth_group(user_name, group_name):
     configmap = v1.read_namespaced_config_map(configmap_name, namespace)
     map_users = yaml.safe_load(configmap.data.get("mapUsers", "[]"))
     # Append the new user to the mapUsers list
-    new_user = {
-        "userarn": f"arn:aws:iam::{get_aws_account_id()}:user/{user_name}",
-        "username": user_name,
-        "groups": [group_name]
-    }
+    new_user = {"userarn": f"arn:aws:iam::{get_aws_account_id()}:user/{user_name}", "username": user_name,
+        "groups": [group_name]}
 
     map_users.append(new_user)
     # Update the mapUsers back into the ConfigMap data
@@ -1126,16 +1179,9 @@ def add_user_to_k8s(user_name):
     config.load_kube_config()
     rbac_api = client.RbacAuthorizationV1Api()
     cluster_role_name = f"{get_cluster_name()}-{user_name}-job-cluster-role"
-    cluster_role = client.V1ClusterRole(
-        metadata=client.V1ObjectMeta(name=cluster_role_name),
-        rules=[
-            client.V1PolicyRule(
-                api_groups=["keda.sh"],
-                resources=["scaledjobs"],
-                verbs=["create", "get", "list", "delete"]
-            )
-        ]
-    )
+    cluster_role = client.V1ClusterRole(metadata=client.V1ObjectMeta(name=cluster_role_name), rules=[
+        client.V1PolicyRule(api_groups=["keda.sh"], resources=["scaledjobs"],
+            verbs=["create", "get", "list", "delete"])])
     try:
         rbac_api.create_cluster_role(body=cluster_role)
     except ApiException as e:
@@ -1147,20 +1193,9 @@ def add_user_to_k8s(user_name):
     group_name = f"{get_cluster_name()}-{user_name}-job-access"
     cluster_role_binding_name = f"{get_cluster_name()}-{user_name}-job-cluster-role-binding"
     # Define ClusterRoleBinding to bind the role to a user
-    cluster_role_binding = client.V1ClusterRoleBinding(
-        metadata=client.V1ObjectMeta(name=cluster_role_binding_name),
-        role_ref=client.V1RoleRef(
-            api_group="rbac.authorization.k8s.io",
-            kind="ClusterRole",
-            name=cluster_role_name
-        ),
-        subjects=[
-            client.RbacV1Subject(
-                kind="Group",
-                name=group_name,
-            )
-        ]
-    )
+    cluster_role_binding = client.V1ClusterRoleBinding(metadata=client.V1ObjectMeta(name=cluster_role_binding_name),
+        role_ref=client.V1RoleRef(api_group="rbac.authorization.k8s.io", kind="ClusterRole", name=cluster_role_name),
+        subjects=[client.RbacV1Subject(kind="Group", name=group_name, )])
     try:
         rbac_api.create_cluster_role_binding(body=cluster_role_binding)
     except ApiException as e:
@@ -1173,12 +1208,8 @@ def add_user_to_k8s(user_name):
     return
 
 
-
 def generate_kubeconfig():
-    eks_client = boto3.client(
-        'eks',
-        region_name=REGION,
-    )
+    eks_client = boto3.client('eks', region_name=REGION, )
 
     cluster_name = get_cluster_name()
     # Retrieve cluster information
@@ -1188,50 +1219,24 @@ def generate_kubeconfig():
     endpoint = cluster_info['endpoint']
     cluster_ca_certificate = cluster_info['certificateAuthority']['data']
 
-
     # Create a kubeconfig dictionary
-    kubeconfig_data = {
-        "apiVersion": "v1",
-        "clusters": [{
-            "cluster": {
-                "server": endpoint,
-                "certificate-authority-data": cluster_ca_certificate
-            },
-            "name": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}'
-        }],
-        "contexts": [{
-            "context": {
-                "cluster": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}',
-                "user": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}'
-            },
-            "name": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}'
-        }],
-        "current-context": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}',
-        "kind": "Config",
-        "users": [{
-            "name": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}',
-            "user": {
-                "exec": {
-                    "apiVersion": "client.authentication.k8s.io/v1beta1",
-                    "command": "aws",
-                    "args": [
-                        "eks",
-                        "get-token",
-                        "--cluster-name", cluster_name
-                    ],
-                    "env": [
-                        {"name": "AWS_DEFAULT_REGION", "value": REGION}
-                    ]
-                }
-            }
-        }]
-    }
+    kubeconfig_data = {"apiVersion": "v1", "clusters": [
+        {"cluster": {"server": endpoint, "certificate-authority-data": cluster_ca_certificate},
+            "name": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}'}], "contexts": [{
+        "context": {"cluster": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}',
+            "user": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}'},
+        "name": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}'}],
+        "current-context": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}', "kind": "Config",
+        "users": [{"name": f'arn:aws:eks:us-east-1:{get_aws_account_id()}:cluster/{cluster_name}', "user": {
+            "exec": {"apiVersion": "client.authentication.k8s.io/v1beta1", "command": "aws",
+                "args": ["eks", "get-token", "--cluster-name", cluster_name],
+                "env": [{"name": "AWS_DEFAULT_REGION", "value": REGION}]}}}]}
     kubeconfig_dir = os.path.expanduser('~/.kube')
     kubeconfig_file = os.path.join(kubeconfig_dir, 'config')
 
     # Create the directory if it does not exist
     if not os.path.exists(kubeconfig_dir):
-        os.makedirs(kubeconfig_dir,exist_ok=True)
+        os.makedirs(kubeconfig_dir, exist_ok=True)
 
     # Load existing kubeconfig or create a new one if it doesn't exist
     if os.path.exists(kubeconfig_file):
@@ -1255,11 +1260,11 @@ def generate_kubeconfig():
     update_or_add_entry(existing_config['users'], kubeconfig_data['users'][0], 'name')
     existing_config['current-context'] = kubeconfig_data['current-context']
 
-
     # Write back to the default kubeconfig file
     with open(kubeconfig_file, 'w') as f:
         yaml.dump(existing_config, f)
 
+
 def create_service_account(name: str, namespace: str, context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
@@ -1273,8 +1278,11 @@ def create_service_account(name: str, namespace: str, context_name: Optional[str
     try:
         v1.create_namespaced_service_account(namespace=namespace, body=service_account)
     except ApiException as e:
-        raise e
-    
+        if e.status == 409:
+            print(f"ServiceAccount '{name}' already exists in namespace '{namespace}'.")
+        else:
+            print(f"Failed to create ServiceAccount '{name}': {e}")
+            raise e
 
 def create_service_account_with_role_arn(name: str, namespace: str, role_arn: str, context_name: Optional[str] = None):
     if not context_name:
@@ -1293,12 +1301,8 @@ def create_service_account_with_role_arn(name: str, namespace: str, role_arn: st
         if e.status == 404:
             # ServiceAccount does not exist; create it
             service_account = client.V1ServiceAccount(
-                metadata=client.V1ObjectMeta(
-                    name=name,
-                    annotations={"eks.amazonaws.com/role-arn": role_arn},
-                    namespace=namespace
-                ),
-            )
+                metadata=client.V1ObjectMeta(name=name, annotations={"eks.amazonaws.com/role-arn": role_arn},
+                    namespace=namespace), )
             try:
                 v1.create_namespaced_service_account(namespace=namespace, body=service_account)
                 print(f"ServiceAccount '{name}' created in namespace '{namespace}'.")
@@ -1310,7 +1314,6 @@ def create_service_account_with_role_arn(name: str, namespace: str, role_arn: st
             raise e
 
 
-
 def create_k8s_role(role_name: str, namespace: str, rules: List[Dict[str, Any]], context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
@@ -1325,9 +1328,15 @@ def create_k8s_role(role_name: str, namespace: str, rules: List[Dict[str, Any]],
     try:
         rbac_api.create_namespaced_role(namespace=namespace, body=role)
     except ApiException as e:
-        raise e
+        if e.status == 409:
+            print(f"Role '{role_name}' already exists in namespace '{namespace}'.")
+        else:
+            print(f"Failed to create Role '{role_name}': {e}")
+            raise e
 
-def create_role_binding(rb_name: str, rb_namespace: str, role_name: str, sa_name: str, sa_namespace: str, context_name: Optional[str] = None):
+
+def create_role_binding(rb_name: str, rb_namespace: str, role_name: str, sa_name: str, sa_namespace: str,
+                        context_name: Optional[str] = None):
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
@@ -1342,7 +1351,11 @@ def create_role_binding(rb_name: str, rb_namespace: str, role_name: str, sa_name
     try:
         rbac_api.create_namespaced_role_binding(namespace=rb_namespace, body=role_binding)
     except ApiException as e:
-        raise e
+        if e.status == 409:
+            print(f"RoleBinding '{rb_name}' already exists in namespace '{rb_namespace}'.")
+        else:
+            print(f"Failed to create RoleBinding '{rb_name}': {e}")
+            raise e
 
 
 def is_existing_scaled_job(job_name: str, namespace: str, context_name: Optional[str] = None):
@@ -1353,13 +1366,8 @@ def is_existing_scaled_job(job_name: str, namespace: str, context_name: Optional
     k8s_api_client = config.new_client_from_config(context=context_name)
     k8s_client = client.CustomObjectsApi(k8s_api_client)
     try:
-        job = k8s_client.get_namespaced_custom_object(
-            group="keda.sh",
-            version="v1alpha1",
-            plural="scaledjobs",
-            name=job_name,
-            namespace=namespace
-        )
+        job = k8s_client.get_namespaced_custom_object(group="keda.sh", version="v1alpha1", plural="scaledjobs",
+            name=job_name, namespace=namespace)
         return True, job
     except ApiException as e:
         if e.status == 404:
@@ -1385,28 +1393,18 @@ def apply_domain_claim(domain: str, namespace: str):
 
     try:
         # Use cluster-scoped methods instead of namespaced ones
-        existing_claim = k8s_client.get_cluster_custom_object(
-            group=group, version=version,
-            plural=plural, name=domain
-        )
+        existing_claim = k8s_client.get_cluster_custom_object(group=group, version=version, plural=plural, name=domain)
         resource_version = existing_claim['metadata']['resourceVersion']
         config_yaml['metadata']['resourceVersion'] = resource_version
-        k8s_client.patch_cluster_custom_object(
-            group=group, version=version,
-            plural=plural, name=domain,
-            body=config_yaml
-        )
+        k8s_client.patch_cluster_custom_object(group=group, version=version, plural=plural, name=domain,
+            body=config_yaml)
         click.echo(f"Updated ClusterDomainClaim for domain {domain}")
     except ApiException as e:
         if e.status == 404:
-            k8s_client.create_cluster_custom_object(
-                group=group, version=version,
-                plural=plural, body=config_yaml
-            )
+            k8s_client.create_cluster_custom_object(group=group, version=version, plural=plural, body=config_yaml)
             click.echo(f"Created ClusterDomainClaim for domain {domain}")
         else:
             raise e
-        
 
 
 def patch_service_account(role_arn, service_account_name, namespace):
@@ -1417,21 +1415,11 @@ def patch_service_account(role_arn, service_account_name, namespace):
     v1 = client.CoreV1Api()
 
     # Define the patch for the ServiceAccount
-    patch_body = {
-        "metadata": {
-            "annotations": {
-                "eks.amazonaws.com/role-arn": role_arn
-            }
-        }
-    }
+    patch_body = {"metadata": {"annotations": {"eks.amazonaws.com/role-arn": role_arn}}}
 
     try:
         # Patch the existing ServiceAccount
-        response = v1.patch_namespaced_service_account(
-            name=service_account_name,
-            namespace=namespace,
-            body=patch_body
-        )
+        response = v1.patch_namespaced_service_account(name=service_account_name, namespace=namespace, body=patch_body)
         print(f"ServiceAccount '{service_account_name}' in namespace '{namespace}' patched successfully with role ARN.")
     except client.exceptions.ApiException as e:
         print(f"Failed to patch ServiceAccount: {e.reason}")
@@ -1445,12 +1433,8 @@ def get_jobs_by_container_image(image: str, namespace: str = "keda") -> list:
     k8s_client = client.CustomObjectsApi(k8s_api_client)
 
     try:
-        scaled_jobs = k8s_client.list_namespaced_custom_object(
-            group="keda.sh",
-            version="v1alpha1",
-            plural="scaledjobs",
-            namespace=namespace
-        )
+        scaled_jobs = k8s_client.list_namespaced_custom_object(group="keda.sh", version="v1alpha1", plural="scaledjobs",
+            namespace=namespace)
         matching_jobs = []
         for job in scaled_jobs.get("items", []):
             containers = job.get("spec", {}).get("jobTargetRef", {}).get("template", {}).get("spec", {}).get(
