@@ -8,10 +8,13 @@ import yaml
 from kubernetes import config, client
 from kubernetes.client import ApiException
 
-from tensorkube.constants import KNATIVE_SERVING_NAMESPACE, CONFIG_FEATURES, TENSORFUSE_NAMESPACES, DEFAULT_NAMESPACE
+from tensorkube.constants import KNATIVE_SERVING_NAMESPACE, CONFIG_FEATURES, TENSORFUSE_NAMESPACES, DEFAULT_NAMESPACE, \
+    GPU_DEPLOYMENT_MIN_MEMORY, GPU_DEPLOYMENT_MIN_CPU, CliColors, ProbeType
+from tensorkube.services.filesystem_service import get_efs_vol_pvc_name
 from tensorkube.services.k8s_service import get_efs_claim_name, get_tensorkube_cluster_context_name
 from tensorkube.services.nydus import get_nydus_runtime_class_name
 from tensorkube.services.nodepool import get_gpu_nodepool
+from tensorkube.services.volumes_service import get_deployment_volume_name, get_deployment_volume_mount_path
 
 
 def get_instance_family_from_gpu_type(gpu_type):
@@ -244,11 +247,45 @@ def apply_tls_virtual_service(service_name: str, domain: str, yaml_file_path: st
         virtual_service_yaml_dict=yaml_dict)
 
 
+
+def set_probe(knative_yaml_dict: Dict, probe_type: ProbeType, probe_dict: Dict) -> Dict:
+    http_get = probe_dict.get('httpGet', None)
+    initial_delay_seconds = probe_dict.get('initialDelaySeconds', None)
+    period_seconds = probe_dict.get('periodSeconds', None)
+    timeout_seconds = probe_dict.get('timeoutSeconds', None)
+    failure_threshold = probe_dict.get('failureThreshold', None)
+    success_threshold = probe_dict.get('successThreshold', None)
+    termination_grace_period_seconds = probe_dict.get('terminationGracePeriodSeconds', None)
+
+    if probe_type == ProbeType.LIVENESS:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe'] = {}
+
+    if http_get:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['httpGet'] = http_get
+    if initial_delay_seconds:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['initialDelaySeconds'] = initial_delay_seconds
+    if period_seconds:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['periodSeconds'] = period_seconds
+    if timeout_seconds:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['timeoutSeconds'] = timeout_seconds
+    if failure_threshold:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['failureThreshold'] = failure_threshold
+    if success_threshold:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['successThreshold'] = success_threshold
+    if termination_grace_period_seconds:
+        knative_yaml_dict['spec']['template']['spec']['containers'][0][f'{probe_type.value}Probe']['terminationGracePeriodSeconds'] = termination_grace_period_seconds
+
+    return knative_yaml_dict
+
+
 def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_project_name: str,
                                       image_tag: str, workdir: str, command: str, gpus: int, gpu_type: str, port: int,
                                       domain: Optional[str], env: Optional[str] = None, cpu: float = 100,
                                       memory: int = 200, min_scale: int = 0, max_scale: int = 3, 
-                                      context_name: Optional[str] = None, secrets: List[str] = [], enable_efs: bool = False, concurrency: int = 100, readiness: Optional[Dict] = None):
+                                      context_name: Optional[str] = None, secrets: List[str] = [],
+                                      enable_efs: bool = False, concurrency: int = 100, readiness: Optional[Dict] = None,
+                                      startup_probe: Optional[Dict] = None, liveness_probe: Optional[Dict] = None,
+                                      volumes: Optional[List[Dict]] = None):
 
     # Load kube config
     if not context_name:
@@ -362,21 +399,32 @@ def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_proj
         if gpus > 0:
             yaml_dict['spec']['template']['spec']['runtimeClassName'] = get_nydus_runtime_class_name()
             if 'nodeSelector' in yaml_dict['spec']['template']['spec']:
-                yaml_dict['spec']['template']['spec']['nodeSelector']['nodepool'] = get_gpu_nodepool()
+                yaml_dict['spec']['template']['spec']['nodeSelector']['karpenter.sh/nodepool'] = get_gpu_nodepool()
             else:
-                yaml_dict['spec']['template']['spec']['nodeSelector'] = {'nodepool': get_gpu_nodepool()}
+                yaml_dict['spec']['template']['spec']['nodeSelector'] = {'karpenter.sh/nodepool': get_gpu_nodepool()}
 
 
     if gpus > 0:
         # by default gpu pods usually use all of available memory and cpu
         # so I am here defining some limits above which the cpu and memory parameters will be used
-        if cpu > 1000:
+        if memory >= GPU_DEPLOYMENT_MIN_MEMORY:
             yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests'][
                 'memory'] = f'{str(int(memory))}M'
-        if memory > 2000:
+        else:
+            click.echo(click.style(
+                f"Memory is not specified or is less than the minimum required for GPU deployment. Setting to minimum: {GPU_DEPLOYMENT_MIN_MEMORY}M.",
+                fg=CliColors.WARNING.value))
             yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests'][
-                'memory'] = f'{str(int(memory))}M'
-        yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests']['cpu'] = f'{str(int(cpu))}m'
+                'memory'] = f'{str(GPU_DEPLOYMENT_MIN_MEMORY)}M'
+        if cpu >= GPU_DEPLOYMENT_MIN_CPU:
+            yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests'][
+                'cpu'] = f'{str(int(cpu))}m'
+        else:
+            click.echo(click.style(
+                f"CPU is not specified or is less than the minimum required for GPU deployment. Setting to minimum: {GPU_DEPLOYMENT_MIN_CPU}m.",
+                fg=CliColors.WARNING.value))
+            yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests'][
+                'cpu'] = f'{str(GPU_DEPLOYMENT_MIN_CPU)}m'
 
         if 'nodeSelector' in yaml_dict['spec']['template']['spec']:
             yaml_dict['spec']['template']['spec']['nodeSelector']['karpenter.k8s.aws/instance-family'] = get_instance_family_from_gpu_type(gpu_type)
@@ -398,8 +446,8 @@ def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_proj
         apply_anti_affinity_for_gpus(yaml_dict)
     if gpus > 2:
         yaml_dict['spec']['template']['metadata']['annotations'][
-            'autoscaling.knative.dev/scale-to-zero-pod-retention-period'] = '15m'
-        yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/scale-down-delay'] = '15m'
+            'autoscaling.knative.dev/scale-to-zero-pod-retention-period'] = '20m'
+        yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/scale-down-delay'] = '20m'
     # apply min scale and max scale arguements
     yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/min-scale'] = str(min_scale)
     yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/max-scale'] = str(max_scale)
@@ -409,7 +457,16 @@ def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_proj
     yaml_dict['spec']['template']['metadata']['annotations']['deploy_time'] = datetime.now().isoformat()
 
     if readiness:
-        yaml_dict['spec']['template']['spec']['containers'][0]['readinessProbe'] = readiness
+        yaml_dict = set_probe(yaml_dict, ProbeType.READINESS, readiness)
+
+    if startup_probe:
+        yaml_dict = set_probe(yaml_dict, ProbeType.STARTUP, startup_probe)
+    else:
+        if readiness:
+            yaml_dict = set_probe(yaml_dict, ProbeType.STARTUP, readiness)
+
+    if liveness_probe:
+        yaml_dict = set_probe(yaml_dict, ProbeType.LIVENESS, liveness_probe)
     
     # set container port
     yaml_dict['spec']['template']['spec']['containers'][0]['ports'] = [{'containerPort': port}]
@@ -417,6 +474,25 @@ def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_proj
         if 'httpGet' in yaml_dict['spec']['template']['spec']['containers'][0]['readinessProbe']:
             yaml_dict['spec']['template']['spec']['containers'][0]['readinessProbe']['httpGet']['port'] = port
 
+    if (startup_probe is None) and (readiness is None):
+        if 'httpGet' in yaml_dict['spec']['template']['spec']['containers'][0]['startupProbe']:
+            yaml_dict['spec']['template']['spec']['containers'][0]['startupProbe']['httpGet']['port'] = port
+
+    if volumes:
+        for volume in volumes:
+            volume_details = {
+                "name" : get_deployment_volume_name(volume['name'], volume['type']),
+                "persistentVolumeClaim":{
+                    "claimName": get_efs_vol_pvc_name(volume['name'], namespace=env)
+                }
+            }
+            volume_mount = {
+                "name": get_deployment_volume_name(volume['name'], volume['type']),
+                "mountPath": get_deployment_volume_mount_path(volume)
+            }
+
+            yaml_dict['spec']['template']['spec']['volumes'].append(volume_details)
+            yaml_dict['spec']['template']['spec']['containers'][0]['volumeMounts'].append(volume_mount)
 
     # enable http redirects if domain is available
     # if domain:
