@@ -544,3 +544,21 @@ def display_dns_records_table(domain: str, validation_records: dict):
     console = Console()
     console.print(table)
     console.print("\n[yellow]Please configure these records in your DNS provider.[/yellow]")
+
+
+def display_efs_volumes(volumes: List[Dict]):
+    """
+    Display EFS volumes in a Rich formatted table
+    Args:
+        volumes: List of EFS volumes
+    """
+    table = Table(title="EFS Volumes")
+    table.add_column("Name", style="green")
+    table.add_column("Volume ID", style="magenta", no_wrap=True)
+    table.add_column("Status", style="cyan")
+
+    for volume in volumes:
+        table.add_row(volume['Name'], volume['FileSystemId'], volume['LifeCycleState'])
+
+    console = Console()
+    console.print(table)
