@@ -11,7 +11,7 @@ import pathspec
 from boto3.s3.transfer import TransferConfig
 from tqdm import tqdm
 
-from tensorkube.constants import REGION, get_cluster_name
+from tensorkube.constants import REGION, get_cluster_name, DEFAULT_NAMESPACE
 from tensorkube.services.utils import human_readable_date, human_readable_size
 
 class S3ObjectInfo(NamedTuple):
@@ -207,7 +207,7 @@ def delete_s3_bucket(bucket_name, region=REGION):
 
 def check_if_a_bucket_type_exists(env_name: Optional[str] = None, type: str = "build"):
     buckets = list_s3_buckets()
-    if env_name:
+    if env_name and env_name != DEFAULT_NAMESPACE:
         prefix = f'{get_cluster_name()}-{env_name}-{type}-bucket-'
     else:
         prefix = f'{get_cluster_name()}-{type}-bucket-'
