@@ -1,5 +1,10 @@
 import boto3
-from tensorkube.constants import REGION
+from tensorkube.constants import REGION, get_cluster_name
+
+
+def get_sqs_access_policy_name():
+    return f"{get_cluster_name()}-sqs-access-policy"
+
 
 def create_sqs_queue(queue_name: str, region: str = REGION) -> str:
     sqs = boto3.client('sqs', region_name=region)
